<?php
/**
 * Ceon URI Mapping Install/Upgrade Script - Installs/Configures Database or upgrades database from
 * an earlier version.
 *
 * @package    ceon_uri_mapping
 * @author     Conor Kerr <zen-cart.uri_mapping@dev.ceon.net>
 * @copyright  Copyright 2008-2009 Ceon
 * @copyright  Copyright 2003-2007 Zen Cart Development Team
 * @copyright  Portions Copyright 2003 osCommerce
 * @link       http://dev.ceon.net/web/zen-cart/uri_mapping
 * @license    http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 * @version    $Id: ceon_uri_mapping_auto_upgrade.php 392 2009-06-27 18:45:09Z Bob $
 */

// {{{ Helper functions

// {{{ ceon_uri_mapping_add_default_index_page_mapping()

/**
 * Adds a default mapping for the home page, so index.php?main_page=index is mapped to the store's
 * main directory (e.g. / or /shop). This mapping will also cause Zen Cart to use / (or /shop etc.)
 * as the link to the home page, instead of index.php?main_page=index.
 *
 * @access   public
 * @author   Conor Kerr <zen-cart.uri_mapping@dev.ceon.net>
 * @return   none
 */
function ceon_uri_mapping_add_default_index_page_mapping()
{
	global $db;
	
	$mapping_exists_language_ids = array();
	$mapping_to_be_added_language_ids = array();
	
	// Check if the mapping already exists for the index page for any of the languages the store
	// uses
	$check_mapping_exists_query = "
		SELECT
			language_id
		FROM
			" . TABLE_CEON_URI_MAPPINGS . "
		WHERE
			uri = '" . zen_db_input(DIR_WS_CATALOG) . "'
		AND
			current_uri = '1'
		AND
			main_page = 'index'
		AND
			query_string_parameters IS NULL
		AND
			associated_db_id IS NULL
		AND
			alternate_uri IS NULL;";
	
	$check_mapping_exists_result = $db->Execute($check_mapping_exists_query);
	
	while (!$check_mapping_exists_result->EOF) {
		$mapping_exists_language_ids[] = $check_mapping_exists_result->fields['language_id'];
		
		$check_mapping_exists_result->MoveNext();
	}
	
	// Build the list of language IDs the mapping needs to be added for
	$languages = zen_get_languages();
	
    for ($i = 0, $n = sizeof($languages); $i < $n; $i++) {
		if (!in_array($languages[$i]['id'], $mapping_exists_language_ids)) {
			$mapping_to_be_added_language_ids[] = $languages[$i]['id'];
		}
	}
	
	$num_mappings_to_be_added = sizeof($mapping_to_be_added_language_ids);
	
	if ($num_mappings_to_be_added > 0) {
		foreach ($mapping_to_be_added_language_ids as $current_language_id) {
			$add_mapping_query = "
				INSERT INTO
					" . TABLE_CEON_URI_MAPPINGS . "
				SET
					uri = '" . zen_db_input(DIR_WS_CATALOG) . "',
					language_id = '" . (int) $current_language_id . "',
					current_uri = '1',
					main_page = 'index',
					date_added = NOW()
					;";
			
			$add_mapping_result = $db->Execute($add_mapping_query);
		}
	}
}

// }}}

// }}}

if (is_null($version)) {
	// Module not installed! Must create tables and default configuration
	
	// Add the URI Mappings table //////////////////////////////////////////////////////////////////
	$create_uri_mappings_table_sql = "
		CREATE TABLE IF NOT EXISTS
			`" . TABLE_CEON_URI_MAPPINGS . "`
			(
			`uri` TEXT NOT NULL,
			`language_id` INT(11) UNSIGNED DEFAULT NULL,
			`current_uri` INT(1) UNSIGNED DEFAULT '0',
			`main_page` VARCHAR(45) NULL,
			`query_string_parameters` VARCHAR(255) DEFAULT NULL,
			`associated_db_id` INT(11) UNSIGNED DEFAULT NULL,
			`alternate_uri` VARCHAR(255) DEFAULT NULL,
			`redirection_type_code` VARCHAR(3) DEFAULT '301',
			`date_added` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00'
			);";
	
	$create_uri_mappings_table_result = $db->Execute($create_uri_mappings_table_sql);
	
	
	// Add the configuration table /////////////////////////////////////////////////////////////////
	$create_uri_mapping_configs_table_sql = "
		CREATE TABLE IF NOT EXISTS
			`" . TABLE_CEON_URI_MAPPING_CONFIGS . "`
			(
			`id` INT UNSIGNED NOT NULL,
			`version` varchar(14) NOT NULL,
			`autogen_new` INT(1) UNSIGNED NOT NULL,
			`whitespace_replacement` INT(1) UNSIGNED NOT NULL,
			`capitalisation` INT(1) UNSIGNED NOT NULL,
			`remove_words` TEXT DEFAULT NULL,
			`char_str_replacements` TEXT DEFAULT NULL,
			`excluded_files` TEXT DEFAULT NULL,
			PRIMARY KEY  (`id`)
		);";
	
	$create_uri_mapping_configs_table_result = $db->Execute($create_uri_mapping_configs_table_sql);
	
	
	// Set up the default configuration ////////////////////////////////////////////////////////////
	$create_default_config_sql = "
		INSERT INTO
			" . TABLE_CEON_URI_MAPPING_CONFIGS . "
			(
			id,
			version,
			autogen_new,
			whitespace_replacement,
			capitalisation,
			remove_words,
			char_str_replacements
			)
		VALUES
			(
			'1',
			'" . CEON_URI_MAPPING_VERSION . "',
			'1',
			'" . CEON_URI_MAPPING_SINGLE_DASH . "',
			'" . CEON_URI_MAPPING_CAPITALISATION_LOWERCASE . "',
			'-, a, an, this, that',
			'=>GBP, $=>USD'
			);";
	
	$create_default_config_result = $db->Execute($create_default_config_sql);
	
	// Add the configuration group and options /////////////////////////////////////////////////////
	$add_config_group_options_sql = "
		INSERT INTO
			" . TABLE_CONFIGURATION_GROUP . "
			(
			configuration_group_title,
			configuration_group_description,
			sort_order,
			visible
			)
		VALUES
			(
			'Ceon URI Mapping (SEO)',
			'Set Ceon URI Mapping (SEO) Options',
			'1',
			'1'
			);";
	
	$add_config_group_options_result = $db->Execute($add_config_group_options_sql);
	
	$add_config_group_options_sql = "
		SELECT
			configuration_group_id  
		FROM
			" . TABLE_CONFIGURATION_GROUP . "
		WHERE
			configuration_group_title = 'Ceon URI Mapping (SEO)';";
	
	$add_config_group_options_result = $db->Execute($add_config_group_options_sql);
	
	if (!$add_config_group_options_result->EOF) {
		$configuration_group_id =
			$add_config_group_options_result->fields['configuration_group_id'];
		
		$add_config_group_options_sql = "
			UPDATE
				" . TABLE_CONFIGURATION_GROUP . "
			SET
				sort_order = '" . $configuration_group_id . "'
			WHERE
				configuration_group_id = '" . $configuration_group_id . "';";
		
		$add_config_group_options_result = $db->Execute($add_config_group_options_sql);
		
		
		$add_config_group_options_sql = "
			INSERT INTO
				" . TABLE_CONFIGURATION . "
				(
				`configuration_title`,
				`configuration_key`,
				`configuration_value`,
				`configuration_description`,
				`configuration_group_id`,
				`sort_order`,
				`set_function`,
				`date_added`
				)
			VALUES
				(
				'Enable/Disable URI Mapping',
				'CEON_URI_MAPPING_ENABLED',
				'1',
				'If enabled, any Categories/Products/Manufacturers/EZ-Pages/other pages which have a static URI Mapping specified for them in the database will use those static URIs instead of the standard Zen Cart dynamically-built URIs.<br /><br />0 = off <br />1 = on',
				'" . $configuration_group_id . "',
				'1',
				'zen_cfg_select_option(array(''0'', ''1''), ',
				NOW()
				);";
		
		$add_config_group_options_result = $db->Execute($add_config_group_options_sql);
	}
	
	$ceon_uri_mapping_just_installed = true;
} else if (substr($version, 0, 1) == '2') {
	// Must update the database!
	
	// Add the new current uri column to the uri mappings table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`current_uri` INT(1) UNSIGNED DEFAULT '0'
		AFTER
			`language_id`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	// Add the new main page column to the uri mappings table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`main_page` VARCHAR(45) NULL
		AFTER
			`current_uri`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	// Add the new associated DB ID column to the uri mappings table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`associated_db_id` INT(11) UNSIGNED DEFAULT NULL
		AFTER
			`main_page`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	// Add the new redirection type code column to the uri mappings table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`redirection_type_code` VARCHAR(3) DEFAULT '301'
		AFTER
			`alternate_uri`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	// Add the new date added column to the uri mappings table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`date_added` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00'
		AFTER
			`redirection_type_code`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	
	// Now modify all existing URI Mappings to use the new format //////////////////////////////////
	$uri_mappings_query = "
		SELECT
			um.*,
			pt.type_handler
		FROM
			" . TABLE_CEON_URI_MAPPINGS . " um
		LEFT JOIN
			" . TABLE_PRODUCTS . " p
		ON
			p.products_id = um.product_id
		LEFT JOIN
			" . TABLE_PRODUCT_TYPES . " pt
		ON
			pt.type_id = p.products_type
		WHERE
			1 = 1;";
	
	$uri_mappings_result = $db->Execute($uri_mappings_query);
	
	while (!$uri_mappings_result->EOF) {
		$main_page = null;
		
		if (!is_null($uri_mappings_result->fields['category_id'])) {
			$main_page = FILENAME_DEFAULT;
			$associated_db_id = $uri_mappings_result->fields['category_id'];
			
			$selection_query_part = "category_id = '" .
				$uri_mappings_result->fields['category_id'] . "'";
		} else if (!is_null($uri_mappings_result->fields['product_id'])) {
			$main_page = $uri_mappings_result->fields['type_handler'] . '_info';
			$associated_db_id = $uri_mappings_result->fields['product_id'];
			
			$selection_query_part = "product_id = '" .
				$uri_mappings_result->fields['product_id'] . "'";
		} else if (!is_null($uri_mappings_result->fields['page_id'])) {
			$main_page = FILENAME_EZPAGES;
			$associated_db_id = $uri_mappings_result->fields['page_id'];
			
			$selection_query_part = "page_id = '" .
				$uri_mappings_result->fields['page_id'] . "'";
		}
		
		if (!is_null($main_page)) {
			$update_record_format_query = "
				UPDATE
					" . TABLE_CEON_URI_MAPPINGS . "
				SET
					current_uri = '1',
					main_page = '" . zen_db_input($main_page) . "',
					associated_db_id = '" . zen_db_input($associated_db_id) . "'
				WHERE
					" . $selection_query_part . ";";
			
			$update_record_format_result = $db->Execute($update_record_format_query);
			
			// Must add the default URI mappings for the other product-related pages
			$uri_mapping = $uri_mappings_result->fields['uri'];
			
			if (substr($uri_mapping, -1) != '/') {
				$uri_mapping .= '/';
			}
			
			$sql_data_array = array(
				'language_id' => $uri_mappings_result->fields['language_id'],
				'current_uri' => 1,
				'associated_db_id' => $associated_db_id,
				'alternate_uri' => (!is_null($uri_mappings_result->fields['alternate_uri']) ?
					$uri_mappings_result->fields['alternate_uri'] : 'null'),
				'redirection_type_code' => $uri_mappings_result->fields['redirection_type_code'],
				'date_added' => $uri_mappings_result->fields['date_added']
				);
			
			$sql_data_array['uri'] = zen_db_prepare_input($uri_mapping .
				CEON_URI_MAPPING_URI_PART_PRODUCT_REVIEWS);
			$sql_data_array['main_page'] = zen_db_prepare_input(FILENAME_PRODUCT_REVIEWS);
			
			zen_db_perform(TABLE_CEON_URI_MAPPINGS, $sql_data_array);
			
			$sql_data_array['uri'] = zen_db_prepare_input($uri_mapping .
				CEON_URI_MAPPING_URI_PART_PRODUCT_REVIEWS_INFO);
			$sql_data_array['main_page'] =
				zen_db_prepare_input(FILENAME_PRODUCT_REVIEWS_INFO);
			
			zen_db_perform(TABLE_CEON_URI_MAPPINGS, $sql_data_array);
			
			$sql_data_array['uri'] = zen_db_prepare_input($uri_mapping .
				CEON_URI_MAPPING_URI_PART_PRODUCT_REVIEWS_WRITE);
			$sql_data_array['main_page'] =
				zen_db_prepare_input(FILENAME_PRODUCT_REVIEWS_WRITE);
			
			zen_db_perform(TABLE_CEON_URI_MAPPINGS, $sql_data_array);
			
			$sql_data_array['uri'] = zen_db_prepare_input($uri_mapping .
				CEON_URI_MAPPING_URI_PART_TELL_A_FRIEND);
			$sql_data_array['main_page'] =
				zen_db_prepare_input(FILENAME_TELL_A_FRIEND);
			
			zen_db_perform(TABLE_CEON_URI_MAPPINGS, $sql_data_array);
		}
		
		$uri_mappings_result->MoveNext();
	}
	
	// Update the date columns to give them a default date
	$update_date_added_query = "
		UPDATE
			" . TABLE_CEON_URI_MAPPINGS . "
		SET
			date_added = NOW()
		WHERE
			1 = 1;";
	
	$update_date_added_result = $db->Execute($update_date_added_query);
	
	// Set the alternate_uri column's values to null where appropriate
	$update_alternate_uri_query = "
		UPDATE
			" . TABLE_CEON_URI_MAPPINGS . "
		SET
			alternate_uri = NULL
		WHERE
			alternate_uri = 'null';";
	
	$update_alternate_uri_result = $db->Execute($update_alternate_uri_query);
	
	// Remove the old columns
	$drop_columns_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		DROP 
			`category_id`,
		DROP 
			`product_id`,
		DROP 
			`page_id`;";
	
	$drop_columns_result = $db->Execute($drop_columns_sql);
	
	// Add the new query string column to the uri mappings table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`query_string_parameters` VARCHAR(255) DEFAULT NULL
		AFTER
			`main_page`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	
	// Now modify all existing URI Mappings to use the new format for other Zen Cart pages /////////
	$uri_mappings_query = "
		SELECT
			um.*
		FROM
			" . TABLE_CEON_URI_MAPPINGS . " um
		WHERE
			alternate_uri LIKE '%index.php?main_page=%';";
	
	$uri_mappings_result = $db->Execute($uri_mappings_query);
	
	while (!$uri_mappings_result->EOF) {
		// Have matched an alternative URI which is a Zen Cart page with preset parameters
		// Extract the values for the main page variable and the query string parameter variable
		$pattern = '/index\.php\?main_page\=([^\&]+)\&?(.*)/';
		
		if (preg_match($pattern, $uri_mappings_result->fields['alternate_uri'], $matches)) {
			// Alternative URI is a Zen Cart page, get any parameters to be set
			$main_page = $matches[1];
			$parameter_string = $matches[2];
			
			if (strlen($parameter_string) == 0) {
				$parameter_string = null;
			}
			
			$update_record_format_query = "
				UPDATE
					" . TABLE_CEON_URI_MAPPINGS . "
				SET
					main_page = '" . zen_db_input($main_page) . "',
					query_string_parameters = " . (is_null($parameter_string) ? 'null' :
						"'" . zen_db_input($parameter_string) . "'") . ",
					alternate_uri = NULL
				WHERE
					alternate_uri = '" . $uri_mappings_result->fields['alternate_uri'] . "'
				AND
					language_id = '" . $uri_mappings_result->fields['language_id'] . "'
				AND
					current_uri = '" . $uri_mappings_result->fields['current_uri'] . "'
				AND
					date_added = '" . $uri_mappings_result->fields['date_added'] . "';";
			
			$update_record_format_result = $db->Execute($update_record_format_query);
		}
		
		$uri_mappings_result->MoveNext();
	}
	
	// Add the default mapping for the home page ///////////////////////////////////////////////////
	ceon_uri_mapping_add_default_index_page_mapping();
	
	// Add the new excluded files column to the uri mappings table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPING_CONFIGS . "
		ADD
			`excluded_files` TEXT DEFAULT NULL
		AFTER
			`char_str_replacements`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
} else if (substr($version, 0, 3) == '3.0') {
	// Add the new query string column to the uri mappings table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`query_string_parameters` VARCHAR(255) DEFAULT NULL
		AFTER
			`main_page`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	
	// Now modify all existing URI Mappings to use the new format for other Zen Cart pages /////////
	$uri_mappings_query = "
		SELECT
			um.*
		FROM
			" . TABLE_CEON_URI_MAPPINGS . " um
		WHERE
			alternate_uri LIKE '%index.php?main_page=%';";
	
	$uri_mappings_result = $db->Execute($uri_mappings_query);
	
	while (!$uri_mappings_result->EOF) {
		// Have matched an alternative URI which is a Zen Cart page with preset parameters
		// Extract the values for the main page variable and the query string parameter variable
		$pattern = '/index\.php\?main_page\=([^\&]+)\&?(.*)/';
		
		if (preg_match($pattern, $uri_mappings_result->fields['alternate_uri'], $matches)) {
			// Alternative URI is a Zen Cart page, get any parameters to be set
			$main_page = $matches[1];
			$parameter_string = $matches[2];
			
			if (strlen($parameter_string) == 0) {
				$parameter_string = null;
			}
			
			$update_record_format_query = "
				UPDATE
					" . TABLE_CEON_URI_MAPPINGS . "
				SET
					main_page = '" . zen_db_input($main_page) . "',
					query_string_parameters = " . (is_null($parameter_string) ? 'null' :
						"'" . zen_db_input($parameter_string) . "'") . ",
					alternate_uri = NULL
				WHERE
					alternate_uri = '" . $uri_mappings_result->fields['alternate_uri'] . "'
				AND
					language_id = '" . $uri_mappings_result->fields['language_id'] . "'
				AND
					current_uri = '" . $uri_mappings_result->fields['current_uri'] . "'
				AND
					date_added = '" . $uri_mappings_result->fields['date_added'] . "';";
			
			$update_record_format_result = $db->Execute($update_record_format_query);
		}
		
		$uri_mappings_result->MoveNext();
	}
	
	// Add the default mapping for the home page ///////////////////////////////////////////////////
	ceon_uri_mapping_add_default_index_page_mapping();
	
	// Add the new excluded files column to the uri mappings table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPING_CONFIGS . "
		ADD
			`excluded_files` TEXT DEFAULT NULL
		AFTER
			`char_str_replacements`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
} else if (substr($version, 0, 3) == '3.2' && $version != '3.2.3') {
	// Add the default mapping for the home page ///////////////////////////////////////////////////
	ceon_uri_mapping_add_default_index_page_mapping();
	
	// Add the new excluded files column to the uri mappings table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPING_CONFIGS . "
		ADD
			`excluded_files` TEXT DEFAULT NULL
		AFTER
			`char_str_replacements`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
} else if (substr($version, 0, 3) == '3.2') {
	// No changes necessary
}


// Update version number of database tables ////////////////////////////////////////////////////////
// Only one config currently supported so ID is hard-coded in following SQL
$update_db_version_number_sql = "
	UPDATE
		" . TABLE_CEON_URI_MAPPING_CONFIGS . "
	SET
		version = '" . CEON_URI_MAPPING_VERSION . "'
	WHERE
		id = '1';";

$update_db_version_number_result = $db->Execute($update_db_version_number_sql);

// Finished!
if (!isset($ceon_uri_mapping_just_installed)) {
	$ceon_uri_mapping_upgraded = true;
}

?>