<?php

/**
 * Ceon URI Mapping Admin and Auto-generation Functions
 *
 * This file contains functions necessary to generate URIs by the Ceon URI Mapping module.
 *
 * @package    ceon_uri_mapping
 * @author     Conor Kerr <zen-cart.uri_mapping@dev.ceon.net>
 * @copyright  Copyright 2008-2009 Ceon
 * @copyright  Copyright 2003-2007 Zen Cart Development Team
 * @copyright  Portions Copyright 2003 osCommerce
 * @link       http://dev.ceon.net/web/zen-cart/uri_mapping
 * @license    http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 * @version    $Id: ceon_uri_mapping.php 392 2009-06-27 18:45:09Z Bob $
 */

/**
 * Load in the String class to handle multibyte characters and transliteration.
 */
require_once(DIR_FS_ADMIN . DIR_WS_CLASSES . 'class.String.php');


// {{{ constants

define('CEON_URI_MAPPING_SINGLE_UNDERSCORE', 1);
define('CEON_URI_MAPPING_SINGLE_DASH', 2);
define('CEON_URI_MAPPING_SINGLE_FULL_STOP', 3);
define('CEON_URI_MAPPING_REMOVE', 4);

define('CEON_URI_MAPPING_CAPITALISATION_LOWERCASE', 1);
define('CEON_URI_MAPPING_CAPITALISATION_AS_IS', 2);
define('CEON_URI_MAPPING_CAPITALISATION_UCFIRST', 3);

// }}}


// {{{ ceon_uri_mapping_autogen_enabled()

/**
 * Checks whether auto-generation of URIs is enabled or disabled.
 *
 * @access  public
 * @return  boolean   Whether auto-generation of URIs is enabled or disabled.
 */
function ceon_uri_mapping_autogen_enabled()
{
	global $db;
	
	if (!isset($autogen_new)) {
		static $autogen_new;
		
		// Only one config currently supported so ID is hard-coded in following SQL
		$load_config_sql = "
			SELECT
				autogen_new
			FROM
				" . TABLE_CEON_URI_MAPPING_CONFIGS . "
			WHERE
				id ='1';";
		
		$load_config_result = $db->Execute($load_config_sql);
		
		if (!$load_config_result->EOF) {
			$autogen_new = $load_config_result->fields['autogen_new'];
		}
	}
	
	if ($autogen_new == 1) {
		return true;
	}
	
	return false;
}

// }}}


// {{{ ceon_uri_mapping_build_category_uri_fields()

/**
 * Builds the input fields for adding/editing URI mappings for categories.
 *
 * @access  public
 * @param   array    $prev_uri_mappings    An array of the current values for the URI mappings, if
 *                                         any.
 * @return  string   The source for the input fields.
 */
function ceon_uri_mapping_build_category_uri_fields($prev_uri_mappings)
{
	global $languages;
	
	$num_prev_uri_mappings = sizeof($prev_uri_mappings);
	
	$num_languages = sizeof($languages);
	
	$uri_mapping_input_fields = zen_draw_separator('pixel_black.gif', '100%', '2');
	
	$uri_mapping_input_fields .= '<table border="0" cellspacing="0" cellpadding="0">' . "\n\t";
	$uri_mapping_input_fields .= '<tr>' . "\n\t\t" .
		'<td rowspan="2" class="main" valign="top" style="width: 10em; padding-top: 0.5em;">';
	$uri_mapping_input_fields .= TEXT_URI_MAPPING_CATEGORY_URI . '</td>' . "\n\t\t" .
		'<td class="main" style="padding-top: 0.5em; padding-bottom: 0;">' . "\n";
	
	for ($i = 0, $n = sizeof($languages); $i < $n; $i++) {
		$uri_mapping_input_fields .= "<p>";
		
		if (!isset($prev_uri_mappings[$languages[$i]['id']])) {
			$prev_uri_mappings[$languages[$i]['id']] = '';
		}
		
		$uri_mapping_input_fields .= zen_draw_hidden_field('prev_uri_mappings[' .
			$languages[$i]['id'] . ']', $prev_uri_mappings[$languages[$i]['id']]);
		
		$uri_mapping_input_fields .= zen_image(DIR_WS_CATALOG_LANGUAGES .
			$languages[$i]['directory'] . '/images/' .
			$languages[$i]['image'], $languages[$i]['name']) . '&nbsp;' .
			zen_draw_input_field('uri_mappings[' . $languages[$i]['id'] . ']',
			$prev_uri_mappings[$languages[$i]['id']], 'size="60"');
		
		$uri_mapping_input_fields .= "</p>\n";
	}
	
	$uri_mapping_input_fields .= '</td>' . "\n\t</tr>\n\t<tr>\n\t\t" .
		'<td class="main" style="padding-top: 1em; padding-bottom: 0.5em;">' . "\n";
	
	$uri_mapping_input_fields .= "<p>";
	
	if (ceon_uri_mapping_autogen_enabled()) {
		if ($num_languages == 1) {
			$autogen_message = TEXT_URI_MAPPING_CATEGORY_URI_AUTOGEN;
		} else {
			$autogen_message = TEXT_URI_MAPPING_CATEGORY_URIS_AUTOGEN;
		}
		if ($num_prev_uri_mappings == 0) {
			$autogen_selected = true;
		} else {
			$autogen_selected = false;
			
			if ($num_prev_uri_mappings == 1) {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_ONE_EXISTING_MAPPING;
			} else if ($num_prev_uri_mappings == $num_languages) {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_ALL_EXISTING_MAPPINGS;
			} else {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_SOME_EXISTING_MAPPINGS;
			}
		}
		
		$uri_mapping_input_fields .=
			zen_draw_checkbox_field('uri_mapping_autogen', '1', $autogen_selected) . ' ' .
			$autogen_message;
	} else {
		$uri_mapping_input_fields .= TEXT_URI_MAPPING_URI_AUTOGEN_DISABLED;
	}
	
	$uri_mapping_input_fields .= "</p>";
	
	$uri_mapping_input_fields .= "\n\t\t</td>\n\t</tr>\n</table>\n";
	
	$uri_mapping_input_fields .= zen_draw_separator('pixel_black.gif', '100%', '2');
	
	return $uri_mapping_input_fields;
}

/// }}}


// {{{ ceon_uri_mapping_build_product_uri_fields()

/**
 * Builds the input fields for adding/editing URI mappings for products.
 *
 * @access  public
 * @param   array    $prev_uri_mappings    An array of the previous values for the URI mappings, if
 *                                         any.
 * @param   array    $uri_mappings         An array of the current values for the URI mappings, if
 *                                         any.
 * @param   boolean  $uri_mapping_autogen  If data has been submitted has the auto-generation
 *                                         checkbox been ticked?
 * @return  string   The source for the input fields.
 */
function ceon_uri_mapping_build_product_uri_fields($prev_uri_mappings, $uri_mappings,
	$uri_mapping_autogen)
{
	global $languages;
	
	$num_uri_mappings = sizeof($uri_mappings);
	
	$num_languages = sizeof($languages);
	
	$uri_mapping_input_fields .= '<tr>
            <td colspan="2">' . zen_draw_separator('pixel_trans.gif', '1', '10') . '</td>
          </tr>
          <tr>
            <td colspan="2">' . zen_draw_separator('pixel_black.gif', '100%', '2') . '</td>
          </tr>
          <tr>
            <td class="main" valign="top" style="padding-top: 0.5em;">' . TEXT_URI_MAPPING_PRODUCT_URI . '</td>
            <td class="main" style="padding-top: 0.5em; padding-bottom: 0.5em;">' . "\n";
	
	for ($i = 0, $n = sizeof($languages); $i < $n; $i++) {
		$uri_mapping_input_fields .= "<p>";
		
		if (!isset($prev_uri_mappings[$languages[$i]['id']])) {
			$prev_uri_mappings[$languages[$i]['id']] = '';
		}
		if (!isset($uri_mappings[$languages[$i]['id']])) {
			$uri_mappings[$languages[$i]['id']] = '';
		}
		
		$uri_mapping_input_fields .= zen_draw_hidden_field('prev_uri_mappings[' .
			$languages[$i]['id'] . ']', $prev_uri_mappings[$languages[$i]['id']]);
		
		$uri_mapping_input_fields .= zen_image(DIR_WS_CATALOG_LANGUAGES .
			$languages[$i]['directory'] . '/images/' .
			$languages[$i]['image'], $languages[$i]['name']) . '&nbsp;' .
			zen_draw_input_field('uri_mappings[' . $languages[$i]['id'] . ']',
			$uri_mappings[$languages[$i]['id']], 'size="60"');
		
		$uri_mapping_input_fields .= "</p>\n";
	}
	
	$uri_mapping_input_fields .= "<p>";
	
	if (ceon_uri_mapping_autogen_enabled()) {
		if ($num_languages == 1) {
			$autogen_message = TEXT_URI_MAPPING_PRODUCT_URI_AUTOGEN;
		} else {
			$autogen_message = TEXT_URI_MAPPING_PRODUCT_URIS_AUTOGEN;
		}
		if ($num_uri_mappings == 0) {
			$autogen_selected = true;
		} else {
			$autogen_selected = false;
			
			if ($num_uri_mappings == 1) {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_ONE_EXISTING_MAPPING;
			} else if ($num_uri_mappings == $num_languages) {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_ALL_EXISTING_MAPPINGS;
			} else {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_SOME_EXISTING_MAPPINGS;
			}
		}
		
		if (!$autogen_selected && $uri_mapping_autogen) {
			$autogen_selected = true;
		}
		
		$uri_mapping_input_fields .=
			zen_draw_checkbox_field('uri_mapping_autogen', '1', $autogen_selected) . ' ' .
			$autogen_message;
	} else {
		$uri_mapping_input_fields .= TEXT_URI_MAPPING_URI_AUTOGEN_DISABLED;
	}
	
	$uri_mapping_input_fields .= "</p>";
	
	$uri_mapping_input_fields .= "\n\t\t</td>\n\t</tr>";
	
	$uri_mapping_input_fields .= '<tr>
            <td colspan="2">' . zen_draw_separator('pixel_black.gif', '100%', '2') . '</td>
          </tr>';
	
	return $uri_mapping_input_fields;
}

/// }}}


// {{{ ceon_uri_mapping_build_product_copy_uri_fields()

/**
 * Builds the input fields for copying URI mappings for products being copied.
 *
 * @access  public
 * @param   integer    $product_id   The ID of the product being copied.
 * @return  string|boolean   The source for the input fields or false if no input fields necessary.
 */
function ceon_uri_mapping_build_product_copy_uri_fields($product_id)
{
	global $db, $languages;
	
	if (!isset($languages) || !is_array($languages)) {
		$languages = zen_get_languages();
	}
	
	$num_languages = sizeof($languages);
	
	// Does this product have any existing mappings?
	$product_has_mappings = false;
	
	$existing_product_uri_mappings_sql = "
		SELECT
			uri
		FROM
			" . TABLE_CEON_URI_MAPPINGS . "
		WHERE
			(
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_GENERAL_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_BOOK_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_FREE_SHIPPING_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_MUSIC_INFO) . "'
			)
		AND
			associated_db_id = '" . (int) $product_id . "';";
	
	$existing_product_uri_mappings_result = $db->Execute($existing_product_uri_mappings_sql);
	
	if (!$existing_product_uri_mappings_result->EOF) {
		$product_has_mappings = true;
	}
	
	$autogen_enabled = ceon_uri_mapping_autogen_enabled();
	
	if ($product_has_mappings || $autogen_enabled) {
		$uri_mapping_input_fields = zen_draw_separator('pixel_black.gif', '100%', '2');
		
		$uri_mapping_input_fields .= "<p>" . TEXT_URI_MAPPING_DUPLICATE_PRODUCT_URI_MAPPING .
			'<br />';
		
		// Default to auto-generating URIs for product's copy
		$autogen_selected = $autogen_enabled;
		$copy_selected = (($product_has_mappings && !$autogen_enabled) ? true : false);
		
		if ($autogen_enabled) {
			if ($num_languages == 1) {
				$autogen_message = TEXT_URI_MAPPING_COPY_AUTOGEN_URI;
			} else {
				$autogen_message = TEXT_URI_MAPPING_COPY_AUTOGEN_URIS;
			}
			$uri_mapping_input_fields .=
				zen_draw_radio_field('uri_mapping', 'autogen', $autogen_selected) . ' ' .
				$autogen_message . '<br />';
		}
		
		if ($product_has_mappings) {
			if ($num_languages == 1) {
				$copy_message = TEXT_URI_MAPPING_COPY_EXISTING_MAPPING;
			} else {
				$copy_message = TEXT_URI_MAPPING_COPY_EXISTING_MAPPINGS;
			}
			$uri_mapping_input_fields .=
				zen_draw_radio_field('uri_mapping', 'copy', $copy_selected) . ' ' .
				$copy_message . '<br />';
		}
		
		if ($autogen_enabled && !$product_has_mappings) {
			if ($num_languages == 1) {
				$ignore_message = TEXT_URI_MAPPING_COPY_PRODUCT_DONT_AUTOGEN_URI;
			} else {
				$ignore_message = TEXT_URI_MAPPING_COPY_PRODUCT_DONT_AUTOGEN_URIS;
			}
		} else if (!$autogen_enabled && $product_has_mappings) {
			if ($num_languages == 1) {
				$ignore_message = TEXT_URI_MAPPING_COPY_PRODUCT_DONT_COPY_URI;
			} else {
				$ignore_message = TEXT_URI_MAPPING_COPY_PRODUCT_DONT_COPY_URIS;
			}
		} else {
			if ($num_languages == 1) {
				$ignore_message = TEXT_URI_MAPPING_COPY_PRODUCT_DONT_AUTOGEN_COPY_URI;
			} else {
				$ignore_message = TEXT_URI_MAPPING_COPY_PRODUCT_DONT_AUTOGEN_COPY_URIS;
			}
		}
		
		$uri_mapping_input_fields .=
			zen_draw_radio_field('uri_mapping', 'ignore', false) . ' ' . $ignore_message;
		
		$uri_mapping_input_fields .= "</p>";
		
		$uri_mapping_input_fields .= zen_draw_separator('pixel_black.gif', '100%', '2');
		
		return $uri_mapping_input_fields;
	}
	
	return false;
}

/// }}}


// {{{ ceon_uri_mapping_build_product_move_uri_fields()

/**
 * Builds the input fields for dealing with URI mappings for products being moved.
 *
 * @access  public
 * @param   integer    $product_id   The ID of the product being moved.
 * @return  string|boolean   The source for the input fields or false if no input fields necessary.
 */
function ceon_uri_mapping_build_product_move_uri_fields($product_id)
{
	global $db, $languages;
	
	if (!isset($languages) || !is_array($languages)) {
		$languages = zen_get_languages();
	}
	
	$num_languages = sizeof($languages);
	
	// Does this product have any existing mappings?
	$product_has_mappings = false;
	
	$existing_product_uri_mappings_sql = "
		SELECT
			uri
		FROM
			" . TABLE_CEON_URI_MAPPINGS . "
		WHERE
			(
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_GENERAL_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_BOOK_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_FREE_SHIPPING_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_MUSIC_INFO) . "'
			)
		AND
			associated_db_id = '" . (int) $product_id . "';";
	
	$existing_product_uri_mappings_result = $db->Execute($existing_product_uri_mappings_sql);
	
	if (!$existing_product_uri_mappings_result->EOF) {
		$product_has_mappings = true;
	}
	
	$autogen_enabled = ceon_uri_mapping_autogen_enabled();
	
	if ($product_has_mappings || $autogen_enabled) {
		$uri_mapping_input_fields = zen_draw_separator('pixel_black.gif', '100%', '2');
		
		$uri_mapping_input_fields .= "<p>" . TEXT_URI_MAPPING_MOVE_PRODUCT_URI_MAPPING .
			'<br />';
		
		// Default to auto-generating URIs for product's new location
		$autogen_selected = $autogen_enabled;
		$move_selected = (($product_has_mappings && !$autogen_enabled) ? true : false);
		
		if ($autogen_enabled) {
			if ($num_languages == 1) {
				$autogen_message = TEXT_URI_MAPPING_MOVE_AUTOGEN_URI;
			} else {
				$autogen_message = TEXT_URI_MAPPING_MOVE_AUTOGEN_URIS;
			}
			$uri_mapping_input_fields .=
				zen_draw_radio_field('uri_mapping', 'autogen', $autogen_selected) . ' ' .
				$autogen_message . '<br />';
		}
		
		if ($product_has_mappings) {
			if ($num_languages == 1) {
				$move_message = TEXT_URI_MAPPING_MOVE_USE_EXISTING_MAPPING;
			} else {
				$move_message = TEXT_URI_MAPPING_MOVE_USE_EXISTING_MAPPINGS;
			}
			$uri_mapping_input_fields .=
				zen_draw_radio_field('uri_mapping', 'use_existing', $move_selected) . ' ' .
				$move_message . '<br />';
		}
		
		if ($autogen_enabled && !$product_has_mappings) {
			if ($num_languages == 1) {
				$ignore_message = TEXT_URI_MAPPING_MOVE_PRODUCT_DONT_AUTOGEN_URI;
			} else {
				$ignore_message = TEXT_URI_MAPPING_MOVE_PRODUCT_DONT_AUTOGEN_URIS;
			}
		} else if (!$autogen_enabled && $product_has_mappings) {
			if ($num_languages == 1) {
				$ignore_message = TEXT_URI_MAPPING_MOVE_PRODUCT_DONT_MOVE_URI;
			} else {
				$ignore_message = TEXT_URI_MAPPING_MOVE_PRODUCT_DONT_MOVE_URIS;
			}
		} else {
			if ($num_languages == 1) {
				$ignore_message = TEXT_URI_MAPPING_MOVE_PRODUCT_DONT_AUTOGEN_MOVE_URI;
			} else {
				$ignore_message = TEXT_URI_MAPPING_MOVE_PRODUCT_DONT_AUTOGEN_MOVE_URIS;
			}
		}
		
		$uri_mapping_input_fields .=
			zen_draw_radio_field('uri_mapping', 'drop_existing', false) . ' ' . $ignore_message;
		
		$uri_mapping_input_fields .= "</p>";
		
		$uri_mapping_input_fields .= zen_draw_separator('pixel_black.gif', '100%', '2');
		
		return $uri_mapping_input_fields;
	}
	
	return false;
}

/// }}}


// {{{ ceon_uri_mapping_build_ez_page_uri_fields()

/**
 * Builds the input fields for adding/editing URI mappings for EZ-Pages.
 *
 * @access  public
 * @param   array    $prev_uri_mappings    An array of the previous values for the URI mappings, if
 *                                         any.
 * @param   array    $uri_mappings         An array of the current values for the URI mappings, if
 *                                         any.
 * @param   boolean  $uri_mapping_autogen  If data has been submitted has the auto-generation
 *                                         checkbox been ticked?
 * @return  string   The source for the input fields.
 */
function ceon_uri_mapping_build_ez_page_uri_fields($prev_uri_mappings, $uri_mappings,
	$uri_mapping_autogen)
{
	global $languages;
	
	$num_uri_mappings = sizeof($uri_mappings);
	
	$num_languages = sizeof($languages);
	
	$uri_mapping_input_fields .= '<tr>
            <td colspan="2">' . zen_draw_separator('pixel_trans.gif', '1', '10') . '</td>
          </tr>
          <tr>
            <td colspan="2">' . zen_draw_separator('pixel_black.gif', '100%', '2') . '</td>
          </tr>
          <tr>
            <td class="main" valign="top" style="padding-top: 0.5em;">' . TEXT_URI_MAPPING_PRODUCT_URI . '</td>
            <td class="main" style="padding-top: 0.5em; padding-bottom: 0.5em;">' . "\n";
	
	for ($i = 0, $n = sizeof($languages); $i < $n; $i++) {
		$uri_mapping_input_fields .= "<p>";
		
		if (!isset($prev_uri_mappings[$languages[$i]['id']])) {
			$prev_uri_mappings[$languages[$i]['id']] = '';
		}
		if (!isset($uri_mappings[$languages[$i]['id']])) {
			$uri_mappings[$languages[$i]['id']] = '';
		}
		
		$uri_mapping_input_fields .= zen_draw_hidden_field('prev_uri_mappings[' .
			$languages[$i]['id'] . ']', $prev_uri_mappings[$languages[$i]['id']]);
		
		$uri_mapping_input_fields .= zen_image(DIR_WS_CATALOG_LANGUAGES .
			$languages[$i]['directory'] . '/images/' .
			$languages[$i]['image'], $languages[$i]['name']) . '&nbsp;' .
			zen_draw_input_field('uri_mappings[' . $languages[$i]['id'] . ']',
			$uri_mappings[$languages[$i]['id']], 'size="60"');
		
		$uri_mapping_input_fields .= "</p>\n";
	}
	
	$uri_mapping_input_fields .= "<p>";
	
	if (ceon_uri_mapping_autogen_enabled()) {
		if ($num_languages == 1) {
			$autogen_message = TEXT_URI_MAPPING_EZ_PAGE_URI_AUTOGEN;
		} else {
			$autogen_message = TEXT_URI_MAPPING_EZ_PAGE_URIS_AUTOGEN;
		}
		if ($num_uri_mappings == 0) {
			$autogen_selected = true;
		} else {
			$autogen_selected = false;
			
			if ($num_uri_mappings == 1) {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_ONE_EXISTING_MAPPING;
			} else if ($num_uri_mappings == $num_languages) {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_ALL_EXISTING_MAPPINGS;
			} else {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_SOME_EXISTING_MAPPINGS;
			}
		}
		
		if (!$autogen_selected && $uri_mapping_autogen) {
			$autogen_selected = true;
		}
		
		$uri_mapping_input_fields .=
			zen_draw_checkbox_field('uri_mapping_autogen', '1', $autogen_selected) . ' ' .
			$autogen_message;
	} else {
		$uri_mapping_input_fields .= TEXT_URI_MAPPING_URI_AUTOGEN_DISABLED;
	}
	
	$uri_mapping_input_fields .= "</p>";
	
	$uri_mapping_input_fields .= "\n\t\t</td>\n\t</tr>";
	
	$uri_mapping_input_fields .= '<tr>
            <td colspan="2">' . zen_draw_separator('pixel_black.gif', '100%', '2') . '</td>
          </tr>';
	
	return $uri_mapping_input_fields;
}

/// }}}


// {{{ ceon_uri_mapping_build_manufacturer_uri_fields()

/**
 * Builds the input fields for adding/editing URI mappings for manufacturers.
 *
 * @access  public
 * @param   array    $prev_uri_mappings    An array of the current values for the URI mappings, if
 *                                         any.
 * @return  string   The source for the input fields.
 */
function ceon_uri_mapping_build_manufacturer_uri_fields($prev_uri_mappings)
{
	global $languages;
	
	$num_prev_uri_mappings = sizeof($prev_uri_mappings);
	
	$num_languages = sizeof($languages);
	
	$uri_mapping_input_fields = zen_draw_separator('pixel_black.gif', '100%', '2');
	
	$uri_mapping_input_fields .= '<table border="0" cellspacing="0" cellpadding="0">' . "\n\t";
	$uri_mapping_input_fields .= '<tr>' . "\n\t\t" .
		'<td rowspan="2" class="main" valign="top" style="width: 10em; padding-top: 0.5em;">';
	$uri_mapping_input_fields .= TEXT_URI_MAPPING_MANUFACTURER_URI . '</td>' . "\n\t\t" .
		'<td class="main" style="padding-top: 0.5em; padding-bottom: 0;">' . "\n";
	
	for ($i = 0, $n = sizeof($languages); $i < $n; $i++) {
		$uri_mapping_input_fields .= "<p>";
		
		if (!isset($prev_uri_mappings[$languages[$i]['id']])) {
			$prev_uri_mappings[$languages[$i]['id']] = '';
		}
		
		$uri_mapping_input_fields .= zen_draw_hidden_field('prev_uri_mappings[' .
			$languages[$i]['id'] . ']', $prev_uri_mappings[$languages[$i]['id']]);
		
		$uri_mapping_input_fields .= zen_image(DIR_WS_CATALOG_LANGUAGES .
			$languages[$i]['directory'] . '/images/' .
			$languages[$i]['image'], $languages[$i]['name']) . '&nbsp;' .
			zen_draw_input_field('uri_mappings[' . $languages[$i]['id'] . ']',
			$prev_uri_mappings[$languages[$i]['id']], 'size="60"');
		
		$uri_mapping_input_fields .= "</p>\n";
	}
	
	$uri_mapping_input_fields .= '</td>' . "\n\t</tr>\n\t<tr>\n\t\t" .
		'<td class="main" style="padding-top: 1em; padding-bottom: 0.5em;">' . "\n";
	
	$uri_mapping_input_fields .= "<p>";
	
	if (ceon_uri_mapping_autogen_enabled()) {
		if ($num_languages == 1) {
			$autogen_message = TEXT_URI_MAPPING_MANUFACTURER_URI_AUTOGEN;
		} else {
			$autogen_message = TEXT_URI_MAPPING_MANUFACTURER_URIS_AUTOGEN;
		}
		if ($num_prev_uri_mappings == 0) {
			$autogen_selected = true;
		} else {
			$autogen_selected = false;
			
			if ($num_prev_uri_mappings == 1) {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_ONE_EXISTING_MAPPING;
			} else if ($num_prev_uri_mappings == $num_languages) {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_ALL_EXISTING_MAPPINGS;
			} else {
				$autogen_message .= '<br />' . TEXT_URI_MAPPING_URI_AUTOGEN_SOME_EXISTING_MAPPINGS;
			}
		}
		
		$uri_mapping_input_fields .=
			zen_draw_checkbox_field('uri_mapping_autogen', '1', $autogen_selected) . ' ' .
			$autogen_message;
	} else {
		$uri_mapping_input_fields .= TEXT_URI_MAPPING_URI_AUTOGEN_DISABLED;
	}
	
	$uri_mapping_input_fields .= "</p>";
	
	$uri_mapping_input_fields .= "\n\t\t</td>\n\t</tr>\n</table>\n";
	
	$uri_mapping_input_fields .= zen_draw_separator('pixel_black.gif', '100%', '2');
	
	return $uri_mapping_input_fields;
}

/// }}}


// {{{ ceon_uri_mapping_autogen_category_or_product_uri()

/**
 * Generates a URI for a category or a product, for the specified language.
 *
 * @access  public
 * @param   integer    $id             The ID of the category/product (used if the details in the
 *                                     database are guaranteed to be up to date).
 * @param   string     $type           Whether the ID corresponds to a category or a product.
 * @param   integer    $parent_category_id   The ID of the parent category (used if the details in 
 *                                           the database could be out of date as new information
 *                                           is being submitted when the URI is being generated).
 * @param   string     $name                 The name of category/product (used if the details in 
 *                                           the database could be out of date as new information
 *                                           is being submitted when the URI is being generated).
 * @param   integer    $language_code  The ISO 639 language code of the language.
 * @param   integer    $language_id    The Zen Cart language ID for the language.
 * @return  string   The auto-generated URI for the category/product and language.
 */
function ceon_uri_mapping_autogen_category_or_product_uri($id, $type, $parent_category_id,
	$name, $language_code, $language_id)
{
	global $db;
	
	// Get the complete path to this category/product, or the parent category if this is a new
	// category/product
	if (!is_null($id)) {
		$category_and_product_path_array =
			ceon_uri_mapping_get_category_or_product_path($id, $type, $language_id);
	} else {
		$category_and_product_path_array =
			ceon_uri_mapping_get_category_or_product_path($parent_category_id, 'category',
			$language_id);
	}
	
	$category_and_product_path_array = array_reverse($category_and_product_path_array);
	
	if (is_null($id) || sizeof($category_and_product_path_array) == 0) {
		// Must add the new category/product's name to the path array
		$category_and_product_path_array[] = $name;
	}
	
	// Must not generate URIs for any category or product which has no name.. would conflict with
	// parent category (manual mapping can be used if that behaviour is required).
	foreach ($category_and_product_path_array as $category_or_product_name) {
		if (String::length($category_or_product_name) == 0 || $category_or_product_name == '/' ||
			$category_or_product_name == '\\') {
			return '';
		}
	}
	
	// Format and transliterate the name of each part of the path
	// Load the configuration settings for the formatting (if not already loaded)
	if (!isset($whitespace_replacement)) {
		$load_config_sql = "
			SELECT
				whitespace_replacement,
				capitalisation,
				remove_words,
				char_str_replacements
			FROM
				" . TABLE_CEON_URI_MAPPING_CONFIGS . " 
			WHERE
				id = '1';";
		
		$load_config_result = $db->Execute($load_config_sql);
		
		if ($load_config_result->EOF) {
			
		} else {
			static $whitespace_replacement;
			$whitespace_replacement = $load_config_result->fields['whitespace_replacement'];
			
			static $capitalisation;
			$capitalisation = $load_config_result->fields['capitalisation'];
			
			static $remove_words;
			$remove_words = $load_config_result->fields['remove_words'];
			
			// Remove any spaces from between the words
			$remove_words = String::regexpReplace($remove_words, '/\s/', '');
			
			// Escape any special characters in the selections of words
			$remove_words = String::regexpReplace($remove_words,
				'/([\/\-\\\!\?\$\^\[\]\|\*\.\(\)\{\}\=\<\>\:])/U', '\\\$1');
			
			// Get the list of escaped words - @TODO is this multibyte safe?
			$remove_words = explode(',', $remove_words);
			
			if (sizeof($remove_words) == 1 && (is_null($remove_words[0]) ||
					strlen($remove_words[0]) == 0)) {
				$remove_words = array();
			}
			
			static $char_str_replacements;
			$char_str_replacements = array();
			
			$char_str_replacements_string = $load_config_result->fields['char_str_replacements'];
			
			$char_str_replacement_pairs = explode(',', $char_str_replacements_string);
			
			for ($i = 0, $n = sizeof($char_str_replacement_pairs); $i < $n; $i++) {
				$current_char_str_replacement = explode('=>', $char_str_replacement_pairs[$i]);
				
				if (sizeof($current_char_str_replacement) == 2) {
					// Escape any special characters in the string to be replaced
					$current_char_str_replacement[0] = String::regexpReplace(
						$current_char_str_replacement[0],
						'/([\/\-\\\!\?\$\^\[\]\|\*\.\(\)\{\}\=\<\>\:])/U', '\\\$1');
					
					// Remove any spaces surrounding the string to be replaced
					$current_char_str_replacement[0] = trim($current_char_str_replacement[0]);
					
					$char_str_replacements[] = array(
						'char_str' => $current_char_str_replacement[0],
						'replacement' => $current_char_str_replacement[1]
						);
				}
			}
		}
	}
	
	for ($i = 0, $n = sizeof($category_and_product_path_array); $i < $n; $i++) {
		// Convert the case of the category/product name according to the configuration setting
		switch ($capitalisation) {
			case CEON_URI_MAPPING_CAPITALISATION_LOWERCASE:
				$category_and_product_path_array[$i] =
					String::toLowercase($category_and_product_path_array[$i]);
				break;
			case CEON_URI_MAPPING_CAPITALISATION_UCFIRST:
				$category_and_product_path_array[$i] =
					String::toUCWords($category_and_product_path_array[$i]);
				break;
		}
		
		// Remove unwanted words
		if (sizeof($remove_words) > 0) {
			// Build a pattern to remove words surrounded by spaces
			$remove_words_pattern = '';
			foreach ($remove_words as $remove_word) {
				// Set word to be removed in the pattern by wrapping it with spaces
				$remove_words_pattern .= '\s' . $remove_word . '\s|';
			}
			$remove_words_pattern = String::substr($remove_words_pattern, 0,
				String::length($remove_words_pattern) - 1);
			
			// Remove the words surrounded by spaces, replacing them with a single space
			$category_and_product_path_array[$i] = String::regexpReplace(
				$category_and_product_path_array[$i], '/' . $remove_words_pattern . '/i', ' ');
			
			// Build a pattern to remove words at the start of the string
			$remove_words_pattern = '';
			foreach ($remove_words as $remove_word) {
				// Set word to be removed in the pattern by wrapping it with spaces
				$remove_words_pattern .= '^' . $remove_word . '\s|';
			}
			$remove_words_pattern = String::substr($remove_words_pattern, 0,
				String::length($remove_words_pattern) - 1);
			
			// Remove any word which is at the start of the string
			$category_and_product_path_array[$i] = String::regexpReplace(
				$category_and_product_path_array[$i], '/' . $remove_words_pattern . '/i', '');
			
			// Build a pattern to remove any word which is at the end of the string
			$remove_words_pattern = '';
			foreach ($remove_words as $remove_word) {
				// Set word to be removed in the pattern by wrapping it with spaces
				$remove_words_pattern .= '\s' . $remove_word . '$|';
			}
			$remove_words_pattern = String::substr($remove_words_pattern, 0,
				String::length($remove_words_pattern) - 1);
			
			// Remove any word which is at the end of the string
			$category_and_product_path_array[$i] = String::regexpReplace(
				$category_and_product_path_array[$i], '/' . $remove_words_pattern . '/i', '');
		}
		
		// Replace specified characters/strings
		if (sizeof($char_str_replacements) > 0) {
			foreach ($char_str_replacements as $char_str_replacement) {
				// Remove the words surrounded by spaces, replacing them with a single space
				$category_and_product_path_array[$i] = String::regexpReplace(
					$category_and_product_path_array[$i],
					'/' . $char_str_replacement['char_str'] . '/i',
					$char_str_replacement['replacement']);
			}
		}
		
		// Convert the category/product name to English ASCII as that's all that's permitted in URIs
		$category_and_product_path_array[$i] =
			String::transliterate($category_and_product_path_array[$i], CHARSET, $language_code);
		
		// Remove illegal characters
		$category_and_product_path_array[$i] = preg_replace('/[^a-zA-Z0-9\.\-_\s]/', '',
			$category_and_product_path_array[$i]);
		
		// Convert double whitespace/tabs etc. to single space
		$category_and_product_path_array[$i] = preg_replace('/\s+/', ' ',
			$category_and_product_path_array[$i]);
		
		// Remove any starting or trailing whitespace
		$category_and_product_path_array[$i] = trim($category_and_product_path_array[$i]);
		
		// Once again convert the case of the category/product name according to the configuration
		// setting. This must be repeated as the transliteration could have left some letters in 
		// place which may need to be converted. As the string is now ASCII, a simple conversion can
		// be used.
		switch ($capitalisation) {
			case CEON_URI_MAPPING_CAPITALISATION_LOWERCASE:
				$category_and_product_path_array[$i] =
					strtolower($category_and_product_path_array[$i]);
				break;
			case CEON_URI_MAPPING_CAPITALISATION_UCFIRST:
				$category_and_product_path_array[$i] =
					ucwords($category_and_product_path_array[$i]);
				break;
		}
		
		// Convert whitespace to configured character (or remove it)
		switch ($whitespace_replacement) {
			case CEON_URI_MAPPING_SINGLE_UNDERSCORE:
				$whitespace_replacement_char = '_';
				break;
			case CEON_URI_MAPPING_SINGLE_DASH:
				$whitespace_replacement_char = '-';
				break;
			case CEON_URI_MAPPING_SINGLE_FULL_STOP:
				$whitespace_replacement_char = '.';
				break;
			case CEON_URI_MAPPING_REMOVE:
				$whitespace_replacement_char = '';
				break;
		}
		$category_and_product_path_array[$i] = preg_replace('/\s/', $whitespace_replacement_char,
			$category_and_product_path_array[$i]);
	}
	
	// Implode the path array into a URI
	$uri = implode('/', $category_and_product_path_array);
	
	// Prepend the URI with the store's directory (if any), otherwise add a "root" slash
	if (strlen(DIR_WS_CATALOG) > 0) {
		$uri = DIR_WS_CATALOG . $uri;
	} else {
		$uri = '/' . $uri;
	}
	
	return $uri;
}

// }}}


// {{{ ceon_uri_mapping_get_category_path()

/**
 * Looks up the hierarchy of the parent categories for the given category/product.
 *
 * @access  public
 * @param   integer    $id                 The ID of the category/product.
 * @param   string     $for                Either "category" or "product".
 * @param   integer    $language_id        The ID of the language of the category names to look up.
 * @param   array      $categories_array   Used internally as part of recursion process.
 * @return  array    A hierarchical array of the parent categories (if any) for the given category/
 *                   product, from the "leaf" product/category back to the "root/top" category.
 */
function ceon_uri_mapping_get_category_or_product_path($id, $for, $language_id,
	$categories_array = '')
{
	global $db;
	
	if (!is_array($categories_array)) {
		$categories_array = array();
	}
	
	if ($id == 0) {
		return $categories_array;
	}
	
	if ($for == 'product') {
		$master_category = $db->Execute("
			SELECT
				pd.products_name,
				p.master_categories_id
			FROM
				" . TABLE_PRODUCTS . " p
			LEFT JOIN
				" . TABLE_PRODUCTS_DESCRIPTION . " pd
			ON
				pd.products_id = p.products_id
			WHERE
				p.products_id = '" . (int) $id . "'
			AND
				pd.language_id = '" . (int) $language_id . "';");
		
		if (!$master_category->EOF) {
			$categories_array[] = $master_category->fields['products_name'];
			
			if ($master_category->fields['master_categories_id'] == '0') {
				// Product uses root/top category
			} else {
				$category = $db->Execute("
					SELECT
						cd.categories_name,
						c.parent_id
					FROM
						" . TABLE_CATEGORIES . " c,
						" . TABLE_CATEGORIES_DESCRIPTION . " cd
					WHERE
						c.categories_id = '" . (int) $master_category->fields['master_categories_id'] . "'
					AND
						c.categories_id = cd.categories_id
					AND
						cd.language_id = '" . (int) $language_id . "'");
				
				$categories_array[] = $category->fields['categories_name'];
				
				if ((zen_not_null($category->fields['parent_id'])) &&
					($category->fields['parent_id'] != '0')) {
						$categories_array = ceon_uri_mapping_get_category_or_product_path(
							$category->fields['parent_id'], 'category', $language_id,
							$categories_array);
				}
			}
		}
	} else if ($for == 'category') {
		$category = $db->Execute("
			SELECT
				cd.categories_name,
				c.parent_id
			FROM
				" . TABLE_CATEGORIES . " c,
				" . TABLE_CATEGORIES_DESCRIPTION . " cd
			WHERE
				c.categories_id = '" . (int) $id . "'
			AND
				c.categories_id = cd.categories_id
			AND
				cd.language_id = '" . (int) $language_id . "'");
		
		$categories_array[] = $category->fields['categories_name'];
		
		if ((zen_not_null($category->fields['parent_id'])) &&
			($category->fields['parent_id'] != '0')) {
				$categories_array = ceon_uri_mapping_get_category_or_product_path(
					$category->fields['parent_id'], 'category', $language_id, $categories_array);
		}
	}
	
	return $categories_array;
}

// }}}


// {{{ ceon_uri_mapping_autogen_ez_page_uri()

/**
 * Generates a URI for an EZ-Page, for the specified language.
 *
 * @access  public
 * @param   integer    $id             The ID of the EZ-Page (used if the name stored in the
 *                                     database is guaranteed to be up to date).
 * @param   string     $name           The name of EZ-Page (used if the name in the database could
 *                                     be out of date as new information is being submitted
 *                                     when the URI is being generated).
 * @param   integer    $language_code  The ISO 639 language code of the language.
 * @param   integer    $language_id    The Zen Cart language ID for the language.
 * @return  string   The auto-generated URI for the EZ-Page and language.
 */
function ceon_uri_mapping_autogen_ez_page_uri($id, $name, $language_code, $language_id)
{
	global $db;
	
	if (!is_null($id)) {
		// Load name from database
		$ez_page_name_result = $db->Execute("
			SELECT
				pages_title
			FROM
				" . TABLE_EZPAGES . "
			WHERE
				pages_id = '" . (int) $id . "'
			AND
				languages_id = '" . (int) $language_id . "'");
		
		$ez_page_name = $ez_page_name_result->fields['pages_title'];
	} else {
		// Simply use name supplied
		$ez_page_name = $name;
	}
	
	// Must not generate URIs for any EZ-Page which has no name!
	if (String::length($ez_page_name) == 0 || $ez_page_name == '/' || $ez_page_name == '\\') {
		return '';
	}
	
	// Format and transliterate the name 
	// Load the configuration settings for the formatting (if not already loaded)
	if (!isset($whitespace_replacement)) {
		$load_config_sql = "
			SELECT
				whitespace_replacement,
				capitalisation,
				remove_words,
				char_str_replacements
			FROM
				" . TABLE_CEON_URI_MAPPING_CONFIGS . " 
			WHERE
				id = '1';";
		
		$load_config_result = $db->Execute($load_config_sql);
		
		if ($load_config_result->EOF) {
			
		} else {
			static $whitespace_replacement;
			$whitespace_replacement = $load_config_result->fields['whitespace_replacement'];
			
			static $capitalisation;
			$capitalisation = $load_config_result->fields['capitalisation'];
			
			static $remove_words;
			$remove_words = $load_config_result->fields['remove_words'];
			
			// Remove any spaces from between the words
			$remove_words = String::regexpReplace($remove_words, '/\s/', '');
			
			// Escape any special characters in the selections of words
			$remove_words = String::regexpReplace($remove_words,
				'/([\/\-\\\!\?\$\^\[\]\|\*\.\(\)\{\}\=\<\>\:])/U', '\\\$1');
			
			// Get the list of escaped words - @TODO is this multibyte safe?
			$remove_words = explode(',', $remove_words);
			
			if (sizeof($remove_words) == 1 && (is_null($remove_words[0]) ||
					strlen($remove_words[0]) == 0)) {
				$remove_words = array();
			}
			
			static $char_str_replacements;
			$char_str_replacements = array();
			
			$char_str_replacements_string = $load_config_result->fields['char_str_replacements'];
			
			$char_str_replacement_pairs = explode(',', $char_str_replacements_string);
			
			for ($i = 0, $n = sizeof($char_str_replacement_pairs); $i < $n; $i++) {
				$current_char_str_replacement = explode('=>', $char_str_replacement_pairs[$i]);
				
				if (sizeof($current_char_str_replacement) == 2) {
					// Escape any special characters in the string to be replaced
					$current_char_str_replacement[0] = String::regexpReplace(
						$current_char_str_replacement[0],
						'/([\/\-\\\!\?\$\^\[\]\|\*\.\(\)\{\}\=\<\>\:])/U', '\\\$1');
					
					// Remove any spaces surrounding the string to be replaced
					$current_char_str_replacement[0] = trim($current_char_str_replacement[0]);
					
					$char_str_replacements[] = array(
						'char_str' => $current_char_str_replacement[0],
						'replacement' => $current_char_str_replacement[1]
						);
				}
			}
		}
	}
	
	// Convert the case of the EZ-Page name according to the configuration setting
	switch ($capitalisation) {
		case CEON_URI_MAPPING_CAPITALISATION_LOWERCASE:
			$ez_page_name = String::toLowercase($ez_page_name);
			break;
		case CEON_URI_MAPPING_CAPITALISATION_UCFIRST:
			$ez_page_name = String::toUCWords($ez_page_name);
			break;
	}
	
	// Remove unwanted words
	if (sizeof($remove_words) > 0) {
		// Build a pattern to remove words surrounded by spaces
		$remove_words_pattern = '';
		foreach ($remove_words as $remove_word) {
			// Set word to be removed in the pattern by wrapping it with spaces
			$remove_words_pattern .= '\s' . $remove_word . '\s|';
		}
		$remove_words_pattern = String::substr($remove_words_pattern, 0,
			String::length($remove_words_pattern) - 1);
		
		// Remove the words surrounded by spaces, replacing them with a single space
		$ez_page_name = String::regexpReplace(
			$ez_page_name, '/' . $remove_words_pattern . '/i', ' ');
		
		
		// Build a pattern to remove words at the start of the string
		$remove_words_pattern = '';
		foreach ($remove_words as $remove_word) {
			// Set word to be removed in the pattern by wrapping it with spaces
			$remove_words_pattern .= '^' . $remove_word . '\s|';
		}
		$remove_words_pattern = String::substr($remove_words_pattern, 0,
			String::length($remove_words_pattern) - 1);
		
		// Remove any word which is at the start of the string
		$ez_page_name = String::regexpReplace(
			$ez_page_name, '/' . $remove_words_pattern . '/i', '');
		
		// Build a pattern to remove any word which at the end of the string
		$remove_words_pattern = '';
		foreach ($remove_words as $remove_word) {
			// Set word to be removed in the pattern by wrapping it with spaces
			$remove_words_pattern .= '\s' . $remove_word . '$|';
		}
		$remove_words_pattern = String::substr($remove_words_pattern, 0,
			String::length($remove_words_pattern) - 1);
		
		// Remove any word which is at the end of the string
		$ez_page_name = String::regexpReplace(
			$ez_page_name, '/' . $remove_words_pattern . '/i', '');
	}
	
	// Replace specified characters/strings
	if (sizeof($char_str_replacements) > 0) {
		foreach ($char_str_replacements as $char_str_replacement) {
			// Remove the words surrounded by spaces, replacing them with a single space
			$category_and_product_path_array[$i] = String::regexpReplace(
				$category_and_product_path_array[$i],
				'/' . $char_str_replacement['char_str'] . '/i',
				$char_str_replacement['replacement']);
		}
	}
	
	// Convert the EZ-Page name to English ASCII as that's all that's permitted in URIs
	$ez_page_name = String::transliterate($ez_page_name, CHARSET, $language_code);
	
	// Remove illegal characters
	$ez_page_name = preg_replace('/[^a-zA-Z0-9\.\-_\s]/', '', $ez_page_name);
	
	// Convert double whitespace/tabs etc. to single space
	$ez_page_name = preg_replace('/\s+/', ' ', $ez_page_name);
	
	// Remove any starting or trailing whitespace
	$ez_page_name = trim($ez_page_name);
	
	// Once again convert the case of the EZ-Page name according to the configuration setting. 
	// This must be repeated as the transliteration could have left some letters in place which 
	// may need to be converted. As the string is now ASCII, a simple conversion can be used.
	switch ($capitalisation) {
		case CEON_URI_MAPPING_CAPITALISATION_LOWERCASE:
			$ez_page_name = strtolower($ez_page_name);
			break;
		case CEON_URI_MAPPING_CAPITALISATION_UCFIRST:
			$ez_page_name = ucwords($ez_page_name);
			break;
	}
	
	// Convert whitespace to configured character (or remove it)
	switch ($whitespace_replacement) {
		case CEON_URI_MAPPING_SINGLE_UNDERSCORE:
			$whitespace_replacement_char = '_';
			break;
		case CEON_URI_MAPPING_SINGLE_DASH:
			$whitespace_replacement_char = '-';
			break;
		case CEON_URI_MAPPING_SINGLE_FULL_STOP:
			$whitespace_replacement_char = '.';
			break;
		case CEON_URI_MAPPING_REMOVE:
			$whitespace_replacement_char = '';
			break;
	}
	$ez_page_name = preg_replace('/\s/', $whitespace_replacement_char, $ez_page_name);
	
	$uri = $ez_page_name;
	
	// Prepend the URI with the store's directory (if any), otherwise add a "root" slash
	if (strlen(DIR_WS_CATALOG) > 0) {
		$uri = DIR_WS_CATALOG . $uri;
	} else {
		$uri = '/' . $uri;
	}
	
	return $uri;
}

// }}}


// {{{ ceon_uri_mapping_autogen_manufacturer_uri()

/**
 * Generates a URI for a manufacturer, for the specified language.
 *
 * @access  public
 * @param   integer    $id             The ID of the manufacturer (used if the name stored in the
 *                                     database is guaranteed to be up to date).
 * @param   string     $name           The name of manufacturer (used if the name in the database 
 *                                     could be out of date as new information is being submitted
 *                                     when the URI is being generated).
 * @param   integer    $language_code  The ISO 639 language code of the language.
 * @param   integer    $language_id    The Zen Cart language ID for the language.
 * @return  string   The auto-generated URI for the manufacturer and language.
 */
function ceon_uri_mapping_autogen_manufacturer_uri($id, $name, $language_code, $language_id)
{
	global $db;
	
	if (!is_null($id)) {
		// Load name from database
		$manufacturer_name_result = $db->Execute("
			SELECT
				manufacturers_name
			FROM
				" . TABLE_MANUFACTURERS . "
			WHERE
				manufacturers_id = '" . (int) $id . "';");
		
		$manufacturer_name = $manufacturer_name_result->fields['manufacturers_name'];
	} else {
		// Simply use name supplied
		$manufacturer_name = $name;
	}
	
	// Must not generate URIs for any manufacturer which has no name!
	if (String::length($manufacturer_name) == 0 || $manufacturer_name == '/' ||
			$manufacturer_name == '\\') {
		return '';
	}
	
	// Format and transliterate the name 
	// Load the configuration settings for the formatting (if not already loaded)
	if (!isset($whitespace_replacement)) {
		$load_config_sql = "
			SELECT
				whitespace_replacement,
				capitalisation,
				remove_words,
				char_str_replacements
			FROM
				" . TABLE_CEON_URI_MAPPING_CONFIGS . " 
			WHERE
				id = '1';";
		
		$load_config_result = $db->Execute($load_config_sql);
		
		if ($load_config_result->EOF) {
			
		} else {
			static $whitespace_replacement;
			$whitespace_replacement = $load_config_result->fields['whitespace_replacement'];
			
			static $capitalisation;
			$capitalisation = $load_config_result->fields['capitalisation'];
			
			static $remove_words;
			$remove_words = $load_config_result->fields['remove_words'];
			
			// Remove any spaces from between the words
			$remove_words = String::regexpReplace($remove_words, '/\s/', '');
			
			// Escape any special characters in the selections of words
			$remove_words = String::regexpReplace($remove_words,
				'/([\/\-\\\!\?\$\^\[\]\|\*\.\(\)\{\}\=\<\>\:])/U', '\\\$1');
			
			// Get the list of escaped words - @TODO is this multibyte safe?
			$remove_words = explode(',', $remove_words);
			
			if (sizeof($remove_words) == 1 && (is_null($remove_words[0]) ||
					strlen($remove_words[0]) == 0)) {
				$remove_words = array();
			}
			
			static $char_str_replacements;
			$char_str_replacements = array();
			
			$char_str_replacements_string = $load_config_result->fields['char_str_replacements'];
			
			$char_str_replacement_pairs = explode(',', $char_str_replacements_string);
			
			for ($i = 0, $n = sizeof($char_str_replacement_pairs); $i < $n; $i++) {
				$current_char_str_replacement = explode('=>', $char_str_replacement_pairs[$i]);
				
				if (sizeof($current_char_str_replacement) == 2) {
					// Escape any special characters in the string to be replaced
					$current_char_str_replacement[0] = String::regexpReplace(
						$current_char_str_replacement[0],
						'/([\/\-\\\!\?\$\^\[\]\|\*\.\(\)\{\}\=\<\>\:])/U', '\\\$1');
					
					// Remove any spaces surrounding the string to be replaced
					$current_char_str_replacement[0] = trim($current_char_str_replacement[0]);
					
					$char_str_replacements[] = array(
						'char_str' => $current_char_str_replacement[0],
						'replacement' => $current_char_str_replacement[1]
						);
				}
			}
		}
	}
	
	// Convert the case of the manufacturer name according to the configuration setting
	switch ($capitalisation) {
		case CEON_URI_MAPPING_CAPITALISATION_LOWERCASE:
			$manufacturer_name = String::toLowercase($manufacturer_name);
			break;
		case CEON_URI_MAPPING_CAPITALISATION_UCFIRST:
			$manufacturer_name = String::toUCWords($manufacturer_name);
			break;
	}
	
	// Remove unwanted words
	if (sizeof($remove_words) > 0) {
		// Build a pattern to remove words surrounded by spaces
		$remove_words_pattern = '';
		foreach ($remove_words as $remove_word) {
			// Set word to be removed in the pattern by wrapping it with spaces
			$remove_words_pattern .= '\s' . $remove_word . '\s|';
		}
		$remove_words_pattern = String::substr($remove_words_pattern, 0,
			String::length($remove_words_pattern) - 1);
		
		// Remove the words surrounded by spaces, replacing them with a single space
		$manufacturer_name = String::regexpReplace(
			$manufacturer_name, '/' . $remove_words_pattern . '/i', ' ');
		
		
		// Build a pattern to remove words at the start of the string
		$remove_words_pattern = '';
		foreach ($remove_words as $remove_word) {
			// Set word to be removed in the pattern by wrapping it with spaces
			$remove_words_pattern .= '^' . $remove_word . '\s|';
		}
		$remove_words_pattern = String::substr($remove_words_pattern, 0,
			String::length($remove_words_pattern) - 1);
		
		// Remove any word which is at the start of the string
		$manufacturer_name = String::regexpReplace(
			$manufacturer_name, '/' . $remove_words_pattern . '/i', '');
		
		// Build a pattern to remove any word which at the end of the string
		$remove_words_pattern = '';
		foreach ($remove_words as $remove_word) {
			// Set word to be removed in the pattern by wrapping it with spaces
			$remove_words_pattern .= '\s' . $remove_word . '$|';
		}
		$remove_words_pattern = String::substr($remove_words_pattern, 0,
			String::length($remove_words_pattern) - 1);
		
		// Remove any word which is at the end of the string
		$manufacturer_name = String::regexpReplace(
			$manufacturer_name, '/' . $remove_words_pattern . '/i', '');
	}
	
	// Replace specified characters/strings
	if (sizeof($char_str_replacements) > 0) {
		foreach ($char_str_replacements as $char_str_replacement) {
			// Remove the words surrounded by spaces, replacing them with a single space
			$category_and_product_path_array[$i] = String::regexpReplace(
				$category_and_product_path_array[$i],
				'/' . $char_str_replacement['char_str'] . '/i',
				$char_str_replacement['replacement']);
		}
	}
	
	// Convert the manufacturer name to English ASCII as that's all that's permitted in URIs
	$manufacturer_name = String::transliterate($manufacturer_name, CHARSET, $language_code);
	
	// Remove illegal characters
	$manufacturer_name = preg_replace('/[^a-zA-Z0-9\.\-_\s]/', '', $manufacturer_name);
	
	// Convert double whitespace/tabs etc. to single space
	$manufacturer_name = preg_replace('/\s+/', ' ', $manufacturer_name);
	
	// Remove any starting or trailing whitespace
	$manufacturer_name = trim($manufacturer_name);
	
	// Once again convert the case of the manufacturer name according to the configuration setting. 
	// This must be repeated as the transliteration could have left some letters in place which 
	// may need to be converted. As the string is now ASCII, a simple conversion can be used.
	switch ($capitalisation) {
		case CEON_URI_MAPPING_CAPITALISATION_LOWERCASE:
			$manufacturer_name = strtolower($manufacturer_name);
			break;
		case CEON_URI_MAPPING_CAPITALISATION_UCFIRST:
			$manufacturer_name = ucwords($manufacturer_name);
			break;
	}
	
	// Convert whitespace to configured character (or remove it)
	switch ($whitespace_replacement) {
		case CEON_URI_MAPPING_SINGLE_UNDERSCORE:
			$whitespace_replacement_char = '_';
			break;
		case CEON_URI_MAPPING_SINGLE_DASH:
			$whitespace_replacement_char = '-';
			break;
		case CEON_URI_MAPPING_SINGLE_FULL_STOP:
			$whitespace_replacement_char = '.';
			break;
		case CEON_URI_MAPPING_REMOVE:
			$whitespace_replacement_char = '';
			break;
	}
	$manufacturer_name = preg_replace('/\s/', $whitespace_replacement_char, $manufacturer_name);
	
	$uri = $manufacturer_name;
	
	// Prepend the URI with the store's directory (if any), otherwise add a "root" slash
	if (strlen(DIR_WS_CATALOG) > 0) {
		$uri = DIR_WS_CATALOG . $uri;
	} else {
		$uri = '/' . $uri;
	}
	
	return $uri;
}

// }}}


// {{{ ceon_uri_mapping_cleanup_uri_mapping()

/**
 * Ensures that a URI matches the format used by the URI mapping functions.
 *
 * @access  public
 * @param   string     $uri    The URI to be checked/cleaned.
 * @return  string   The checked/cleaned URI.
 */
function ceon_uri_mapping_cleanup_uri_mapping($uri)
{
	// Remove any starting or trailing whitespace
	$uri = trim($uri);
	
	// Replace any backslashes with forward slashes
	$uri = str_replace('\\', '/', $uri);
	
	// Remove illegal characters
	$uri = preg_replace('|[^a-zA-Z0-9\.\-_\/]|', '', $uri);
	
	// Remove any domain specification as all URIs must be relative to the site's root
	$uri = preg_replace('|^http:\/\/[^\/]+|iU', '', $uri);
	
	if (String::length($uri) == 0) {
		return '';
	}
		
	// Prepend the URI with a root slash
	while (String::substr($uri, 0, 1) == '/') {
		$uri = String::substr($uri, 1, String::length($uri) - 1);
	}
	$uri = '/' . $uri;
	
	// Remove any double trailing slashes, matching algorithm only allows for one
	while (String::substr($uri, -2) == '//') {
		$uri = String::substr($uri, 0, String::length($uri) - 1);
	}
		
	return $uri;
}

// }}}


// {{{ ceon_uri_mapping_remove_product_mapping_duplicates()

/**
 * Remove any duplicate URI mappings for a product.
 *
 * @access  public
 * @param   string     $uri    The URI to be checked for duplicates.
 * @param   integer    $product_id  The ID of the product.
 * @return  none
 */
function ceon_uri_mapping_remove_product_mapping_duplicates($base_uri, $product_id)
{
	global $db;
	
	$db->Execute("
		DELETE FROM
		" . TABLE_CEON_URI_MAPPINGS . "
		WHERE
			(
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_GENERAL_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_BOOK_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_FREE_SHIPPING_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_MUSIC_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_WRITE) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_TELL_A_FRIEND) . "'
			)
		AND
			uri = '" . zen_db_input($base_uri) . "'
		AND
			associated_db_id = '" . (int) $product_id . "'
		AND
			current_uri = '0';");
	
	
	if (substr($base_uri, -1) != '/') {
		$base_uri .= '/';
	}
	
	$db->Execute("
		DELETE FROM
		" . TABLE_CEON_URI_MAPPINGS . "
		WHERE
			(
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_GENERAL_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_BOOK_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_FREE_SHIPPING_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_MUSIC_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_WRITE) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_TELL_A_FRIEND) . "'
			)
		AND
			uri = '" . zen_db_input($base_uri) . "'
		AND
			associated_db_id = '" . (int) $product_id . "'
		AND
			current_uri = '0';");
	
	$db->Execute("
		DELETE FROM
		" . TABLE_CEON_URI_MAPPINGS . "
		WHERE
			(
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_GENERAL_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_BOOK_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_FREE_SHIPPING_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_MUSIC_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_WRITE) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_TELL_A_FRIEND) . "'
			)
		AND
			uri = '" . zen_db_input($base_uri . CEON_URI_MAPPING_URI_PART_PRODUCT_REVIEWS) . "'
		AND
			associated_db_id = '" . (int) $product_id . "'
		AND
			current_uri = '0';");
	
	$db->Execute("
		DELETE FROM
		" . TABLE_CEON_URI_MAPPINGS . "
		WHERE
			(
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_GENERAL_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_BOOK_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_FREE_SHIPPING_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_MUSIC_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_WRITE) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_TELL_A_FRIEND) . "'
			)
		AND
			uri = '" . zen_db_input($base_uri . CEON_URI_MAPPING_URI_PART_PRODUCT_REVIEWS_INFO) . "'
		AND
			associated_db_id = '" . (int) $product_id . "'
		AND
			current_uri = '0';");
	
	$db->Execute("
		DELETE FROM
		" . TABLE_CEON_URI_MAPPINGS . "
		WHERE
			(
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_GENERAL_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_BOOK_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_FREE_SHIPPING_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_MUSIC_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_WRITE) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_TELL_A_FRIEND) . "'
			)
		AND
			uri = '" . zen_db_input($base_uri . CEON_URI_MAPPING_URI_PART_PRODUCT_REVIEWS_WRITE) . "'
		AND
			associated_db_id = '" . (int) $product_id . "'
		AND
			current_uri = '0';");
	
	$db->Execute("
		DELETE FROM
		" . TABLE_CEON_URI_MAPPINGS . "
		WHERE
			(
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_GENERAL_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_DOCUMENT_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_BOOK_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_FREE_SHIPPING_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_MUSIC_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_INFO) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_PRODUCT_REVIEWS_WRITE) . "'
			OR
				main_page = '" . zen_db_input(FILENAME_TELL_A_FRIEND) . "'
			)
		AND
			uri = '" . zen_db_input($base_uri . CEON_URI_MAPPING_URI_PART_TELL_A_FRIEND) . "'
		AND
			associated_db_id = '" . (int) $product_id . "'
		AND
			current_uri = '0';");
}

// }}}

?>