<?php
/**
 * Ceon URI Mapping functionality initialisation/processing script
 * 
 * @package    ceon_uri_mapping
 * @author     Conor Kerr <zen-cart.uri_mapping@dev.ceon.net>
 * @copyright  Copyright 2008-2009 Ceon
 * @copyright  Copyright 2003-2007 Zen Cart Development Team
 * @copyright  Portions Copyright 2003 osCommerce
 * @link       http://dev.ceon.net/web/zen-cart/uri_mapping
 * @license    http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 * @version    $Id: init_ceon_uri_mapping.php 392 2009-06-27 18:45:09Z Bob $
 */

if (!defined('IS_ADMIN_FLAG')) {
	die('Illegal Access');
}

// Make sure rewrite rules haven't interfered with value of server variable used by some Zen Cart
// functionality
if ($PHP_SELF == '/') {
	$PHP_SELF = '/index.php';
} else if ($PHP_SELF == DIR_WS_CATALOG) {
	$PHP_SELF = DIR_WS_CATALOG . 'index.php';
}

if (defined('CEON_URI_MAPPING_ENABLED') && CEON_URI_MAPPING_ENABLED == 1) {
	if (!isset($_SERVER['REQUEST_URI'])) {
		print 'Ceon URI Mapping cannot access REQUEST_URI server variable!';
	} else {
		$request_uri = $_SERVER['REQUEST_URI'];
		
		// Remove and store any query string
		$query_string = null;
		$query_pos = strpos($request_uri, '?');
		if ($query_pos !== false) {
			$query_string = substr($request_uri, $query_pos + 1,
				strlen($request_uri) - $query_pos - 1);
			$request_uri = substr($request_uri, 0, $query_pos);
			
			if (isset($_GET[zen_session_name()])) {
				// Remove any session ID from the query string as it certainly won't be matched
				// against in the database!
				$session_parameter_pattern = '/&?' . zen_session_name() . '=[^&]+/i';
				$query_string = preg_replace($session_parameter_pattern, '', $query_string);
			}
		}
		
		// Don't process any excluded files
		// First prepare paths of default exclusions for comparison against the request URI
		$num_excluded_files = sizeof($ceon_uri_mapping_excluded_files);
		
		for ($i = 0; $i < $num_excluded_files; $i++) {
			$ceon_uri_mapping_excluded_files[$i] = strtolower(DIR_WS_CATALOG) .
				$ceon_uri_mapping_excluded_files[$i];
		}
		
		// Now load and prepare the paths of any specific exclusions for the store
		$get_excluded_files_query = "
			SELECT
				excluded_files
			FROM
				" . TABLE_CEON_URI_MAPPING_CONFIGS . "
			WHERE
				id = '1';";
		
		$get_excluded_files_result = $db->Execute($get_excluded_files_query);
		
		if (!$get_excluded_files_result->EOF) {
			$excluded_files_string = $get_excluded_files_result->fields['excluded_files'];
			
			$excluded_files_array = explode(',', $excluded_files_string);
			
			foreach ($excluded_files_array as $excluded_file) {
				$ceon_uri_mapping_excluded_files[] = strtolower(trim($excluded_file));
			}
		}
		
		if (!in_array(strtolower($request_uri), $ceon_uri_mapping_excluded_files)) {
			// Run the language initsystem code here as, for some strange reason, it runs late in
			// the initsystem process and the Ceon URI Mapping module must run before the sanitize
			// initsystem script which normally preceeds the language initsystem code.
			if (!isset($_SESSION['language']) || isset($_GET['language'])) {
				$lng = new language();
				if (isset($_GET['language']) && zen_not_null($_GET['language'])) {
					$lng->set_language($_GET['language']);
				} else {
					if (LANGUAGE_DEFAULT_SELECTOR == 'Browser') {
						$lng->get_browser_language();
					} else {
						$lng->set_language(DEFAULT_LANGUAGE);
					}
				}
				$_SESSION['language'] = (zen_not_null($lng->language['directory']) ? $lng->language['directory'] : 'english');
				$_SESSION['languages_id'] = (zen_not_null($lng->language['id']) ? $lng->language['id'] : 1);
				$_SESSION['languages_code'] = (zen_not_null($lng->language['code']) ? $lng->language['code'] : 'en');
			}
			
			if (!isset($_GET['main_page']) && isset($_SERVER['REQUEST_URI'])) {
				// Standard Zen Cart URI isn't being used or page is possibly the index page
				
				// Check if this page is the index page
				if ($request_uri == DIR_WS_CATALOG || ($request_uri == DIR_WS_CATALOG . '/') ||
						($request_uri == DIR_WS_CATALOG . 'index.php')) {
					$_GET['main_page'] = FILENAME_DEFAULT;
				}
			}
			
			if (!isset($_GET['main_page']) && isset($_SERVER['REQUEST_URI'])) {
				// Standard Zen Cart URI isn't being used, attempt to identify and map URI.
				
				// Attempt to match slight variations of the URI by matching against a regular
				// expression that allows for an optional trailing slash
				// First make sure that request URI has no invalid characters in it
				$uri_to_match = preg_replace('/[^a-zA-Z0-9_\-\.\/%]/', '', $request_uri);
				
				if (substr($uri_to_match, -1) == '/') {
					// Remove the trailing slash to prevent duplication in the regular expression
					$uri_to_match = substr($uri_to_match, 0, strlen($uri_to_match) -1);
				}
				
				$pattern = '^' . $uri_to_match . '\/?$';
				
				$identify_main_page_query = "
					SELECT
						um.language_id,
						um.main_page,
						um.query_string_parameters,
						um.associated_db_id,
						um.alternate_uri,
						um.current_uri,
						um.redirection_type_code
					FROM
						" . TABLE_CEON_URI_MAPPINGS . " um
					WHERE
						um.uri REGEXP '" . zen_db_prepare_input($pattern) . "'
					ORDER BY
						um.language_id,
						um.current_uri DESC,
						um.date_added DESC;";
				
				$identify_main_page_result = $db->Execute($identify_main_page_query);
				
				if (!$identify_main_page_result->EOF) {
					// Have matched the URI!
					
					// Use the mapping which specifically maps the customer's language ID (if
					// possible), otherwise simply use the last match
					while (!$identify_main_page_result->EOF) {
						$language_id = $identify_main_page_result->fields['language_id'];
						$main_page = $identify_main_page_result->fields['main_page'];
						$query_string_parameters =
							$identify_main_page_result->fields['query_string_parameters'];
						$associated_db_id = $identify_main_page_result->fields['associated_db_id'];
						$alternate_uri = $identify_main_page_result->fields['alternate_uri'];
						$current_uri = $identify_main_page_result->fields['current_uri'];
						$redirection_type_code =
							$identify_main_page_result->fields['redirection_type_code'];
						
						if ($language_id == $_SESSION['languages_id']) {
							break;
						}
						
						$identify_main_page_result->MoveNext();
					}
					
					if (!is_null($alternate_uri)) {
						$alternate_uri = trim($alternate_uri);
						if (strlen($alternate_uri) < 2) {
							$alternate_uri = null;
						}
					}
					
					if (!is_null($main_page) && is_null($alternate_uri)) {
						// Have matched a Zen Cart page to initialise
						
						// Is the URI matched for a different language from the default/current
						// language? If so, must use this other language!
						if ($language_id != $_SESSION['languages_id']) {
							$language_info_query = "
								SELECT
									directory,
									code
								FROM
									" . TABLE_LANGUAGES . " 
								WHERE
									languages_id = '" . (int) $language_id . "';";
							
							$language_info_result = $db->Execute($language_info_query);
							
							$_SESSION['language'] = $language_info_result->fields['directory'];
							$_SESSION['languages_id'] = $language_id;
							$_SESSION['languages_code'] = $language_info_result->fields['code'];
						}
						
						if ($current_uri != 1) {
							// This isn't a current URI, is there a current URI to redirect to? Or
							// is this an old URI being kept in the history to avoid broken links?
							if (!is_null($associated_db_id)) {
								$redirection_uri_query = "
									SELECT
										um.uri
									FROM
										" . TABLE_CEON_URI_MAPPINGS . " um
									WHERE
										um.main_page = '" . zen_db_input(zen_db_prepare_input($main_page)) . "'
									AND
										um.associated_db_id = '" . (int) $associated_db_id . "'
									AND
										um.language_id = '" . (int) $_SESSION['languages_id'] . "'
									AND
										um.current_uri = '1';";
							} else if (!is_null($query_string_parameters)) {
								$redirection_uri_query = "
									SELECT
										um.uri
									FROM
										" . TABLE_CEON_URI_MAPPINGS . " um
									WHERE
										um.main_page = '" . zen_db_input(zen_db_prepare_input($main_page)) . "'
									AND
										um.query_string_parameters = '" . zen_db_input(zen_db_prepare_input($query_string)) . "'
									AND
										um.language_id = '" . (int) $_SESSION['languages_id'] . "'
									AND
										um.current_uri = '1';";
							} else {
								$redirection_uri_query = "
									SELECT
										um.uri
									FROM
										" . TABLE_CEON_URI_MAPPINGS . " um
									WHERE
										um.main_page = '" . zen_db_input(zen_db_prepare_input($main_page)) . "'
									AND
										um.associated_db_id IS NULL
									AND
										um.query_string_parameters IS NULL
									AND
										um.language_id = '" . (int) $_SESSION['languages_id'] . "'
									AND
										um.current_uri = '1';";
							}
							
							$redirection_uri_result = $db->Execute($redirection_uri_query);
							
							if (!$redirection_uri_result->EOF) {
								// Have found a current URI mapping for this page, force a redirect
								// to it
								$redirection_uri = $redirection_uri_result->fields['uri'];
								
								// Include any extra info in the GET variables
								if (!is_null($associated_db_id) ||
										is_null($query_string_parameters)) {
									$query_string = '';
									foreach ($_GET as $key => $value) {
										$query_string .= '&' . urlencode($key) . '=' .
											urlencode($value);
									}
									
									if ($query_string != '') {
										$query_string = '?' . substr($query_string, 1,
											strlen($query_string) - 1);
									}
								} else if (!is_null($query_string_parameters)) {
									// Query string variables will be rebuilt after redirecting so
									// no need to append them
									$query_string = '';
								}
								
								header("HTTP/1.1 301 Moved Permanently");
								header("Location: " . $redirection_uri . $query_string);
								zen_exit();
							} else {
								// There is no current URI, this is a URI from the historical
								// database being used to prevent broken links. Force a redirect to
								// standard Zen Cart page
								
								if (!is_null($associated_db_id)) {
									// Must supply the ID information necessary for the correct info
									// to be loaded
									switch ($main_page) {
										case FILENAME_DEFAULT:
											if (!isset($_GET['cPath'])) {
												$_GET['cPath'] = $associated_db_id;
											}
											break;
										case FILENAME_EZPAGES:
										case FILENAME_EZPAGES_POPUP:
											$_GET['id'] = $associated_db_id;
											break;
										default:
											// This is a product related page
											$_GET['products_id'] = $associated_db_id;
											break;
									}
								} else if (!is_null($query_string_parameters)) {
									// Must make sure that all parameters for this URI are loaded
									$query_string_pairs = explode('&', $query_string_parameters);
									
									foreach ($query_string_pairs as $query_string_pair) {
										$parameter_parts = explode('=', $query_string_pair);
										
										// Parameter from database overrides any in query string as
										// this is a historical URI mapping.
										if (sizeof($parameter_parts) == 2) {
											$_GET[$parameter_parts[0]] =
												urldecode($parameter_parts[1]);
										}
									}
								}
								
								if ($request_type == 'SSL') {
									$redirection_uri = DIR_WS_HTTPS_CATALOG;
								} else {
									$redirection_uri = DIR_WS_CATALOG;
								}
								
								$redirection_uri .= 'index.php?main_page=' . $main_page;
								$parameters = zen_get_all_get_params();
								
								if (strlen($parameters) > 0) {
									$redirection_uri .= '&' . $parameters;
								}
								
								if (substr($redirection_uri, 0, 1) != '/') {
									$redirection_uri = '/' . $redirection_uri;
								}
								
								if (substr($redirection_uri, -1) == '&') {
									$redirection_uri = substr($redirection_uri, 0,
										strlen($redirection_uri) - 1);
								}
								
								// Record that the source of redirect was this script, avoiding any
								// unnecessary processing later
								$_SESSION['ceon_uri_mapping_redirected'] = true;
								
								header("HTTP/1.1 301 Moved Permanently");
								header("Location: " . $redirection_uri);
								zen_exit();
							}
						}
						
						// This URI is the current URI for this page
						$_GET['main_page'] = $main_page;
						
						if (!is_null($associated_db_id)) {
							// Page is a category, EZ-Page or a product info page, must initialise
							// the appropriate ID
							if ($main_page == FILENAME_DEFAULT) {
								// This is a category page
								// Must build the full contextual link to the category, so any
								// hierarchy is respected
								$categories = array();
								zen_get_parent_categories($categories, $associated_db_id);
								
								$categories = array_reverse($categories);
								
								$cPath = implode('_', $categories);
								
								if (zen_not_null($cPath)) {
									$cPath .= '_';
								}
								$cPath .= $associated_db_id;
								
								$_GET['cPath'] = $cPath;
							} else if ($main_page == FILENAME_EZPAGES ||
									$main_page == FILENAME_EZPAGES_POPUP) {
								// Have found an EZ-Page. Should it be displayed as a normal page or
								// in a popup?
								if ($main_page == FILENAME_EZPAGES_POPUP ||
										isset($_GET['ezpopup'])) {
									$_GET['main_page'] = FILENAME_EZPAGES_POPUP;
								} else {
									$_GET['main_page'] = FILENAME_EZPAGES;
								}
								$_GET['id'] = $associated_db_id;
							} else {
								// This is a product related page
								$_GET['products_id'] = $associated_db_id;
								
								// Rebuild the cPath variable for this page if it doesn't exist
								if (!isset($_GET['cPath'])) {
									$_GET['cPath'] = zen_get_product_path($_GET['products_id']);
								}
							}
						} else if (!is_null($query_string_parameters)) {
							// Page is a Zen Cart page with preset parameters, must initialise the
							// parameters
							$query_string_pairs = explode('&', $query_string_parameters);
							
							foreach ($query_string_pairs as $query_string_pair) {
								$parameter_parts = explode('=', $query_string_pair);
								
								// Parameter from database overrides any in query string as this is
								// a historical URI mapping.
								if (sizeof($parameter_parts) == 2) {
									$_GET[$parameter_parts[0]] = urldecode($parameter_parts[1]);
								}
							}
						}
					} else if (!is_null($alternate_uri)) {
						// Alternative URI is a full URI to be redirected to
						switch ($redirection_type_code) {
							case 301:
								header("HTTP/1.1 301 Moved Permanently");
								break;
							case 302:
								header("HTTP/1.1 302 Found");
								break;
							case 303:
								header("HTTP/1.1 302 See Other ");
								break;
							case 307:
								header("HTTP/1.1 302 Temporary Redirect");
								break;
						}
						header("Location: " . $alternate_uri);
						zen_exit();
					} else {
						header('HTTP/1.1 404 Not Found');
						$_GET['main_page'] = 'page_not_found';
						
						print "Unable to map identified URI: $request_uri";
					}
				} else if ($uri_to_match . '/' == DIR_WS_CATALOG) {
					// This is the index page and there is no mapping in the database for the index
					// page
					$_GET['main_page'] = 'index';
				} else {
					if (MISSING_PAGE_CHECK == 'On' || MISSING_PAGE_CHECK == 'true') {
						$_GET['main_page'] = 'index';
					} elseif (MISSING_PAGE_CHECK == 'Page Not Found') {
						header('HTTP/1.1 404 Not Found');
						$_GET['main_page'] = 'page_not_found';
					}
				}
			} else if (isset($_GET['main_page']) &&
					isset($_SESSION['ceon_uri_mapping_redirected'])) {
				// This script redirected the customer here so there's no need for any further
				// checks
				unset($_SESSION['ceon_uri_mapping_redirected']);
			} else if (isset($_GET['main_page']) &&
					!isset($_SESSION['ceon_uri_mapping_redirected'])) {
				// This is a standard Zen Cart dynamic URI - redirect it to its mapped URI if there
				// is one
				$redirection_uri = null;
				
				$associated_db_id = null;
				$query_string_to_match = null;
				
				if ($_GET['main_page'] == FILENAME_DEFAULT) {
					// Is this a category?
					if (isset($_GET['cPath']) && !isset($_GET['manufacturers_id'])) {
						// This is a category page - get the URI for the category ID
						$category_parts = explode('_', $_GET['cPath']);
						$category_id = $category_parts[sizeof($category_parts) - 1];
						
						$associated_db_id = $category_id;
					} else if (isset($_GET['manufacturers_id'])) {
						// This is a manufacturer page - get the URI for the manufacturers page if
						// there is one
						$query_string_to_match = 'manufacturers_id=' . $_GET['manufacturers_id'];
					}
				} else if($_GET['main_page'] == FILENAME_DOCUMENT_GENERAL_INFO ||
						$_GET['main_page'] == FILENAME_DOCUMENT_PRODUCT_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_BOOK_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_FREE_SHIPPING_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_MUSIC_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS ||
						$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_INFO ||
						$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_WRITE ||
						$_GET['main_page'] == FILENAME_TELL_A_FRIEND) {
					$associated_db_id = $_GET['products_id'];
				} else if ($_GET['main_page'] == FILENAME_EZPAGES ||
						$_GET['main_page'] == FILENAME_EZPAGES_POPUP) {
					$associated_db_id = $_GET['id'];
				} else if ((sizeof($_GET) > 1 && !isset($_GET[zen_session_name()])) ||
						(isset($_GET[zen_session_name()]) && sizeof($_GET) > 2)) {
					// Check if there's a query string to match against for this Zen Cart page
					// Get the query string
					$query_pos = strpos($_SERVER['REQUEST_URI'], '?');
					if ($query_pos !== false) {
						$query_string_to_match = substr($_SERVER['REQUEST_URI'], $query_pos + 1,
							strlen($_SERVER['REQUEST_URI']) - $query_pos - 1);
						
						// Remove main_page variable from query string
						$query_string_to_match = str_replace('main_page=' . $_GET['main_page'], '',
							$query_string_to_match);
						
						if (substr($query_string_to_match, 0, 1) == '&') {
							$query_string_to_match = substr($query_string_to_match, 1,
								strlen($query_string_to_match) - 1);
						}
						if (substr($query_string_to_match, -1) == '&') {
							$query_string_to_match = substr($query_string_to_match, 0,
								strlen($query_string_to_match) - 1);
						}
						
						if (isset($_GET[zen_session_name()])) {
							// Remove any session ID from the query string as it certainly won't be
							// matched against in the database!
							$session_parameter_pattern = '/&?' . zen_session_name() . '=[^&]+/i';
							$query_string_to_match = preg_replace($session_parameter_pattern, '',
								$query_string_to_match);
						}
					}
				}
				
				if (!is_null($associated_db_id)) {
					$redirection_uri_query = "
						SELECT
							um.uri
						FROM
							" . TABLE_CEON_URI_MAPPINGS . " um
						WHERE
							um.main_page = '" . zen_db_input(zen_db_prepare_input($_GET['main_page'])) . "'
						AND
							um.associated_db_id = '" . (int) $associated_db_id . "'
						AND
							um.language_id = '" . (int) $_SESSION['languages_id'] . "'
						AND
							um.current_uri = '1';";
				} else if (!is_null($query_string_to_match)) {
					$redirection_uri_query = "
						SELECT
							um.uri
						FROM
							" . TABLE_CEON_URI_MAPPINGS . " um
						WHERE
							um.main_page = '" . zen_db_input(zen_db_prepare_input($_GET['main_page'])) . "'
						AND
							um.query_string_parameters = '" . zen_db_input(zen_db_prepare_input($query_string_to_match)) . "'
						AND
							um.language_id = '" . (int) $_SESSION['languages_id'] . "'
						AND
							um.current_uri = '1';";
				} else {
					$redirection_uri_query = "
						SELECT
							um.uri
						FROM
							" . TABLE_CEON_URI_MAPPINGS . " um
						WHERE
							um.main_page = '" . zen_db_input(zen_db_prepare_input($_GET['main_page'])) . "'
						AND
							um.associated_db_id IS NULL
						AND
							um.query_string_parameters IS NULL
						AND
							um.language_id = '" . (int) $_SESSION['languages_id'] . "'
						AND
							um.current_uri = '1';";
				}
				
				$redirection_uri_result = $db->Execute($redirection_uri_query);
				
				if (!$redirection_uri_result->EOF) {
					// Have found a URI mapping for this page, use this as the "current" URI for
					// this page by forcing a redirect to it
					$redirection_uri = $redirection_uri_result->fields['uri'];
					
					// Perform a sanity check to make sure that the URI to be redirected to isn't
					// simply the current URI 
					if ($request_uri != $redirection_uri) {
						// Fine to redirect
						
						// Remove unnecessary variables from query string
						if ($_GET['main_page'] == FILENAME_DEFAULT && isset($_GET['cPath'])) {
							// Remove the category ID from the URI as it will be regenerated when
							// the static URI is loaded
							unset($_GET['cPath']);
						} else if ($_GET['main_page'] == FILENAME_DEFAULT &&
								isset($_GET['manufacturer_id'])) {
							// Remove the manufacturer ID from the URI as it will be regenerated
							// when the static URI is loaded
							unset($_GET['manufacturer_id']);
						} else if($_GET['main_page'] == FILENAME_DOCUMENT_GENERAL_INFO ||
								$_GET['main_page'] == FILENAME_DOCUMENT_PRODUCT_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_BOOK_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_FREE_SHIPPING_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_MUSIC_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS ||
								$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_INFO ||
								$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_WRITE ||
								$_GET['main_page'] == FILENAME_TELL_A_FRIEND) {
							unset($_GET['products_id']);
						} else if ($_GET['main_page'] == FILENAME_EZPAGES ||
								$_GET['main_page'] == FILENAME_EZPAGES_POPUP) {
							unset($_GET['id']);
						}
						
						unset($_GET['main_page']);
						
						if (!is_null($query_string_to_match)) {
							// Remove the query string parameters which matched those in the string
							// in the database as they will be set automatically using the same
							// parameter string associated with the mapped URI in the database -
							// this means they don't have to be displayed to the customer!
							// All other parameters should be passed or additional functionality on
							// the mapped page would not work correctly!
							$query_string_parameter_pairs = explode('&', $query_string_to_match);
							
							foreach ($query_string_parameter_pairs as
									$current_query_string_parameter_pair) {
								$key_and_value = explode('=', $current_query_string_parameter_pair);
								
								unset($_GET[$key_and_value[0]]);
							}
						}
					} else {
						// Don't redirect to the same address!
						$redirection_uri = null;
					}
				} else if (is_null($associated_db_id) && !is_null($query_string_to_match)) {
					// Could this possibly be a Zen Cart page without parameters?
					$redirection_uri_query = "
						SELECT
							um.uri
						FROM
							" . TABLE_CEON_URI_MAPPINGS . " um
						WHERE
							um.main_page = '" . zen_db_input(zen_db_prepare_input($_GET['main_page'])) . "'
						AND
							um.associated_db_id IS NULL
						AND
							um.query_string_parameters IS NULL
						AND
							um.language_id = '" . (int) $_SESSION['languages_id'] . "'
						AND
							um.current_uri = '1';";
					
					$redirection_uri_result = $db->Execute($redirection_uri_query);
					
					if (!$redirection_uri_result->EOF) {
						$redirection_uri = $redirection_uri_result->fields['uri'];
						
						// Perform a sanity check to make sure that the URI to be redirected to
						// isn't simply the current URI 
						if ($request_uri != $redirection_uri) {
							// Fine to redirect, remove unnecessary page identifier from query
							// string
							unset($_GET['main_page']);
						} else {
							// Don't redirect to the same address!
							$redirection_uri = null;
						}
					}
				}
				
				if (!is_null($redirection_uri)) {
					$query_string = '';
					foreach ($_GET as $key => $value) {
						$query_string .= '&' . urlencode($key) . '=' . urlencode($value);
					}
					
					if ($query_string != '') {
						$query_string = '?' . substr($query_string, 1, strlen($query_string) - 1);
					}
					
					header("HTTP/1.1 301 Moved Permanently");
					header("Location: " . $redirection_uri . $query_string);
					zen_exit();
				}
			}
		}
	}
}

?>